"""
PDF文档水印处理器 - 支持.pdf格式添加明水印
"""

import io
import os
from pathlib import Path
from typing import Optional, Union, List, Tuple
from PyPDF2 import PdfReader, PdfWriter
from reportlab.pdfgen import canvas
from reportlab.lib.pagesizes import A4, letter
from reportlab.lib.colors import Color
from reportlab.pdfbase import pdfmetrics
from reportlab.pdfbase.ttfonts import TTFont

from visualwm.core.info import WatermarkInfo
from visualwm.core.style import WatermarkStyle, RiskLevel


class PDFWatermark:
    """
    PDF文档水印处理器
    
    支持在PDF文档中添加明水印，包括：
    - 平铺水印
    - 居中水印
    - 页眉/页脚水印
    - 对角线水印
    
    Features:
        - 支持.pdf格式
        - 保持原PDF结构
        - 支持加密PDF（需要密码）
        - 多种水印模式
        - 批量处理
    """
    
    SUPPORTED_EXTENSIONS = {'.pdf'}
    
    # 字体搜索路径
    FONT_PATHS = [
        os.path.join(os.path.dirname(__file__), "..", "font"),  # SDK内置字体目录
        "/usr/share/fonts/opentype/noto/",
        "/usr/share/fonts/truetype/wqy/",
        "/usr/share/fonts/truetype/",
        "/usr/share/fonts/",
        "/System/Library/Fonts/",
        "C:/Windows/Fonts/",
        "./fonts/",
    ]
    
    # 中文字体文件优先级列表
    CJK_FONT_FILES = [
        "NotoSerifSC-VariableFont_wght.ttf",  # SDK内置字体
        "NotoSansCJK-Regular.ttc",
        "NotoSansSC-Regular.otf",
        "wqy-zenhei.ttc",
        "wqy-microhei.ttc",
        "simhei.ttf",
        "SimHei.ttf",
    ]
    
    def __init__(self, default_style: Optional[WatermarkStyle] = None):
        """
        初始化PDF水印处理器
        
        Args:
            default_style: 默认水印样式
        """
        self.default_style = default_style or WatermarkStyle.default()
        self._font_registered = False
    
    def _register_font(self, font_name: str = "SimHei"):
        """注册中文字体"""
        if self._font_registered:
            return "ChineseFont"
        
        # 优先使用CJK字体文件列表
        for font_file in self.CJK_FONT_FILES:
            for font_path in self.FONT_PATHS:
                full_path = os.path.join(font_path, font_file)
                if os.path.exists(full_path):
                    try:
                        pdfmetrics.registerFont(TTFont("ChineseFont", full_path))
                        self._font_registered = True
                        return "ChineseFont"
                    except Exception as e:
                        continue
        
        # 备用字体列表
        font_files = [
            ("SimHei", ["simhei.ttf", "SimHei.ttf"]),
            ("SimSun", ["simsun.ttc", "SimSun.ttc"]),
            ("STHeiti", ["STHeiti.ttf"]),
            ("WenQuanYi", ["wqy-microhei.ttc", "wenquanyi-microhei.ttc"]),
            ("Noto", ["NotoSansCJK-Regular.ttc", "NotoSansSC-Regular.otf"]),
        ]
        
        for name, files in font_files:
            for font_file in files:
                for font_path in self.FONT_PATHS:
                    full_path = os.path.join(font_path, font_file)
                    if os.path.exists(full_path):
                        try:
                            pdfmetrics.registerFont(TTFont(name, full_path))
                            self._font_registered = True
                            return name
                        except Exception:
                            continue
        
        # 使用默认字体
        return "Helvetica"
    
    def embed(
        self,
        input_path: Union[str, Path],
        output_path: Union[str, Path],
        info: WatermarkInfo,
        style: Optional[WatermarkStyle] = None,
        watermark_type: str = "tile",
        password: Optional[str] = None
    ) -> str:
        """
        在PDF文档中嵌入水印
        
        Args:
            input_path: 输入PDF路径
            output_path: 输出PDF路径
            info: 水印信息
            style: 水印样式
            watermark_type: 水印类型 ("tile", "center", "diagonal", "header_footer")
            password: PDF密码（如果加密）
            
        Returns:
            输出文件路径
        """
        input_path = Path(input_path)
        output_path = Path(output_path)
        style = style or self.default_style
        
        if not input_path.exists():
            raise FileNotFoundError(f"输入文件不存在: {input_path}")
        
        # 读取原PDF
        reader = PdfReader(input_path)
        
        if reader.is_encrypted:
            if password:
                reader.decrypt(password)
            else:
                raise ValueError("PDF已加密，请提供密码")
        
        writer = PdfWriter()
        
        # 为每一页添加水印
        for page_num, page in enumerate(reader.pages):
            # 获取页面尺寸
            page_width = float(page.mediabox.width)
            page_height = float(page.mediabox.height)
            
            # 创建水印PDF
            watermark_pdf = self._create_watermark_page(
                info, style, watermark_type,
                (page_width, page_height)
            )
            
            # 合并水印
            watermark_reader = PdfReader(watermark_pdf)
            watermark_page = watermark_reader.pages[0]
            
            page.merge_page(watermark_page)
            writer.add_page(page)
        
        # 保存
        output_path.parent.mkdir(parents=True, exist_ok=True)
        with open(output_path, 'wb') as f:
            writer.write(f)
        
        return str(output_path)
    
    def _create_watermark_page(
        self,
        info: WatermarkInfo,
        style: WatermarkStyle,
        watermark_type: str,
        page_size: Tuple[float, float]
    ) -> io.BytesIO:
        """
        创建水印页面
        
        Args:
            info: 水印信息
            style: 水印样式
            watermark_type: 水印类型
            page_size: 页面尺寸
            
        Returns:
            水印PDF的BytesIO对象
        """
        packet = io.BytesIO()
        c = canvas.Canvas(packet, pagesize=page_size)
        
        # 注册字体
        font_name = self._register_font(style.font_name)
        
        # 获取水印文本
        watermark_text = self._format_watermark_text(info, style)
        
        # 设置透明度
        c.setFillColor(Color(
            style.color[0] / 255,
            style.color[1] / 255,
            style.color[2] / 255,
            alpha=style.opacity
        ))
        
        # 根据类型绘制水印
        if watermark_type == "tile":
            self._draw_tiled_watermark(c, watermark_text, style, page_size, font_name)
        elif watermark_type == "center":
            self._draw_centered_watermark(c, watermark_text, style, page_size, font_name)
        elif watermark_type == "diagonal":
            self._draw_diagonal_watermark(c, watermark_text, style, page_size, font_name)
        elif watermark_type == "header_footer":
            self._draw_header_footer_watermark(c, watermark_text, style, page_size, font_name)
        
        c.save()
        packet.seek(0)
        
        return packet
    
    def _draw_tiled_watermark(
        self,
        c: canvas.Canvas,
        text: str,
        style: WatermarkStyle,
        page_size: Tuple[float, float],
        font_name: str
    ):
        """绘制平铺水印"""
        width, height = page_size
        
        c.setFont(font_name, style.font_size)
        
        # 计算文本宽度
        text_width = c.stringWidth(text, font_name, style.font_size)
        text_height = style.font_size
        
        # 计算间距
        spacing_x = text_width + style.column_spacing
        spacing_y = text_height + style.line_spacing
        
        # 绘制平铺水印
        y = -spacing_y
        row = 0
        while y < height + spacing_y:
            x = -text_width + (spacing_x // 2 if row % 2 else 0)
            while x < width + text_width:
                c.saveState()
                c.translate(x + text_width / 2, y + text_height / 2)
                c.rotate(style.rotation)
                c.drawString(-text_width / 2, -text_height / 2, text)
                c.restoreState()
                x += spacing_x
            y += spacing_y
            row += 1
    
    def _draw_centered_watermark(
        self,
        c: canvas.Canvas,
        text: str,
        style: WatermarkStyle,
        page_size: Tuple[float, float],
        font_name: str
    ):
        """绘制居中水印"""
        width, height = page_size
        
        c.setFont(font_name, style.font_size * 1.5)  # 居中水印稍大
        
        text_width = c.stringWidth(text, font_name, style.font_size * 1.5)
        
        c.saveState()
        c.translate(width / 2, height / 2)
        c.rotate(style.rotation)
        c.drawString(-text_width / 2, 0, text)
        c.restoreState()
    
    def _draw_diagonal_watermark(
        self,
        c: canvas.Canvas,
        text: str,
        style: WatermarkStyle,
        page_size: Tuple[float, float],
        font_name: str
    ):
        """绘制对角线水印"""
        width, height = page_size
        
        # 使用较大字体
        font_size = style.font_size * 2
        c.setFont(font_name, font_size)
        
        text_width = c.stringWidth(text, font_name, font_size)
        
        # 从左下到右上绘制
        c.saveState()
        c.translate(width / 2, height / 2)
        c.rotate(-45)  # 对角线角度
        c.drawString(-text_width / 2, 0, text)
        c.restoreState()
    
    def _draw_header_footer_watermark(
        self,
        c: canvas.Canvas,
        text: str,
        style: WatermarkStyle,
        page_size: Tuple[float, float],
        font_name: str
    ):
        """绘制页眉页脚水印"""
        width, height = page_size
        
        font_size = style.font_size * 0.6
        c.setFont(font_name, font_size)
        
        text_width = c.stringWidth(text, font_name, font_size)
        
        # 页眉
        c.drawString((width - text_width) / 2, height - style.margin - font_size, text)
        
        # 页脚
        c.drawString((width - text_width) / 2, style.margin, text)
    
    def _format_watermark_text(self, info: WatermarkInfo, style: WatermarkStyle) -> str:
        """格式化水印文本"""
        text = info.to_string(separator=" | ", compact=True)
        
        if style.prefix:
            text = f"{style.prefix} {text}"
        if style.suffix:
            text = f"{text} {style.suffix}"
        
        return text
    
    def create_watermarked_pdf(
        self,
        output_path: Union[str, Path],
        content: str,
        info: WatermarkInfo,
        style: Optional[WatermarkStyle] = None,
        page_size: Tuple[float, float] = A4
    ) -> str:
        """
        创建带水印的新PDF文档
        
        Args:
            output_path: 输出路径
            content: 文档内容
            info: 水印信息
            style: 水印样式
            page_size: 页面尺寸
            
        Returns:
            输出文件路径
        """
        output_path = Path(output_path)
        style = style or self.default_style
        
        # 注册字体
        font_name = self._register_font(style.font_name)
        
        packet = io.BytesIO()
        c = canvas.Canvas(packet, pagesize=page_size)
        
        width, height = page_size
        
        # 设置字体
        c.setFont(font_name, 12)
        
        # 绘制内容
        y = height - 50
        for line in content.split('\n'):
            if y < 50:
                c.showPage()
                c.setFont(font_name, 12)
                y = height - 50
            c.drawString(50, y, line)
            y -= 15
        
        c.save()
        packet.seek(0)
        
        # 保存临时PDF
        temp_path = output_path.with_suffix('.temp.pdf')
        with open(temp_path, 'wb') as f:
            f.write(packet.getvalue())
        
        # 添加水印
        self.embed(temp_path, output_path, info, style)
        
        # 删除临时文件
        os.unlink(temp_path)
        
        return str(output_path)
    
    def extract(
        self,
        pdf_path: Union[str, Path]
    ) -> Optional[WatermarkInfo]:
        """
        从PDF中提取水印信息
        
        Args:
            pdf_path: PDF路径
            
        Returns:
            提取的水印信息
            
        Note:
            PDF水印提取较为复杂，可能需要OCR支持
        """
        reader = PdfReader(pdf_path)
        
        # 尝试从第一页提取文本
        for page in reader.pages:
            text = page.extract_text()
            if text:
                # 查找水印特征
                lines = text.split('\n')
                for line in lines:
                    if "用户:" in line or "工号:" in line:
                        return WatermarkInfo.from_string(line.strip())
        
        return None
    
    def batch_embed(
        self,
        input_dir: Union[str, Path],
        output_dir: Union[str, Path],
        info: WatermarkInfo,
        style: Optional[WatermarkStyle] = None,
        recursive: bool = False
    ) -> List[str]:
        """
        批量添加水印
        
        Args:
            input_dir: 输入目录
            output_dir: 输出目录
            info: 水印信息
            style: 水印样式
            recursive: 是否递归处理
            
        Returns:
            处理的文件列表
        """
        input_dir = Path(input_dir)
        output_dir = Path(output_dir)
        
        processed = []
        
        pattern = "**/*.pdf" if recursive else "*.pdf"
        
        for input_file in input_dir.glob(pattern):
            relative_path = input_file.relative_to(input_dir)
            output_file = output_dir / relative_path
            
            try:
                self.embed(input_file, output_file, info, style)
                processed.append(str(output_file))
            except Exception as e:
                print(f"处理失败 {input_file}: {e}")
        
        return processed
    
    def get_pdf_info(self, pdf_path: Union[str, Path]) -> dict:
        """
        获取PDF信息
        
        Args:
            pdf_path: PDF路径
            
        Returns:
            PDF信息字典
        """
        reader = PdfReader(pdf_path)
        
        info = {
            "pages": len(reader.pages),
            "encrypted": reader.is_encrypted,
            "metadata": {},
        }
        
        if reader.metadata:
            info["metadata"] = {
                "title": reader.metadata.get("/Title", ""),
                "author": reader.metadata.get("/Author", ""),
                "creator": reader.metadata.get("/Creator", ""),
                "producer": reader.metadata.get("/Producer", ""),
            }
        
        # 获取第一页尺寸
        if reader.pages:
            page = reader.pages[0]
            info["width"] = float(page.mediabox.width)
            info["height"] = float(page.mediabox.height)
        
        return info



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
VisualWM 示例代码 - 文本水印
"""

from visualwm import TextWatermark, WatermarkInfo, RiskLevel
from visualwm.text.watermark import (
    TextWatermarkStyle,
    TextWatermarkFormat,
    HTMLWatermark,
    MarkdownWatermark
)


def example_basic_text_watermark():
    """基本文本水印示例"""
    print("=" * 60)
    print("示例1: 基本文本水印（头尾模式）")
    print("=" * 60)
    
    # 创建水印信息
    info = WatermarkInfo(
        username="张三",
        employee_id="EMP001",
        device_id="DEV-2024-001",
        department="研发部",
        auto_fill=True
    )
    
    # 原始文本
    content = """这是一份重要的技术文档。

本文档包含公司核心技术资料，请妥善保管。

第一章：概述
本系统采用微服务架构设计...

第二章：技术细节
系统使用Python语言开发..."""
    
    # 创建文本水印处理器
    wm = TextWatermark()
    
    # 添加水印
    watermarked = wm.embed(content, info)
    
    print("添加水印后的文本:")
    print("-" * 40)
    print(watermarked[:500] + "..." if len(watermarked) > 500 else watermarked)
    print("-" * 40)
    print("✓ 基本文本水印完成")


def example_header_watermark():
    """头部水印示例"""
    print("\n" + "=" * 60)
    print("示例2: 头部水印")
    print("=" * 60)
    
    info = WatermarkInfo(
        username="李四",
        employee_id="EMP002",
        auto_fill=True
    )
    
    style = TextWatermarkStyle(
        format=TextWatermarkFormat.HEADER,
        include_border=True,
        border_char="*"
    )
    
    content = "这是文档内容..."
    
    wm = TextWatermark()
    watermarked = wm.embed(content, info, style)
    
    print("头部水印:")
    print(watermarked)
    print("✓ 头部水印完成")


def example_footer_watermark():
    """尾部水印示例"""
    print("\n" + "=" * 60)
    print("示例3: 尾部水印")
    print("=" * 60)
    
    info = WatermarkInfo(
        username="王五",
        employee_id="EMP003",
        auto_fill=True
    )
    
    style = TextWatermarkStyle(
        format=TextWatermarkFormat.FOOTER
    )
    
    content = "这是文档内容..."
    
    wm = TextWatermark()
    watermarked = wm.embed(content, info, style)
    
    print("尾部水印:")
    print(watermarked)
    print("✓ 尾部水印完成")


def example_inline_watermark():
    """行内水印示例"""
    print("\n" + "=" * 60)
    print("示例4: 行内水印")
    print("=" * 60)
    
    info = WatermarkInfo(
        username="赵六",
        employee_id="EMP004",
        auto_fill=True
    )
    
    style = TextWatermarkStyle(
        format=TextWatermarkFormat.INLINE,
        repeat_interval=3,  # 每3行添加一次水印
        bracket_left="《",
        bracket_right="》"
    )
    
    content = """第一行内容
第二行内容
第三行内容
第四行内容
第五行内容
第六行内容
第七行内容
第八行内容
第九行内容"""
    
    wm = TextWatermark()
    watermarked = wm.embed(content, info, style)
    
    print("行内水印（每3行一次）:")
    print(watermarked)
    print("✓ 行内水印完成")


def example_margin_watermark():
    """边距水印示例"""
    print("\n" + "=" * 60)
    print("示例5: 边距水印")
    print("=" * 60)
    
    info = WatermarkInfo(
        username="孙七",
        employee_id="EMP005",
        auto_fill=True
    )
    
    style = TextWatermarkStyle(
        format=TextWatermarkFormat.MARGIN,
        line_prefix="| "
    )
    
    content = """这是第一行
这是第二行
这是第三行"""
    
    wm = TextWatermark()
    watermarked = wm.embed(content, info, style)
    
    print("边距水印:")
    print(watermarked)
    print("✓ 边距水印完成")


def example_high_risk_text_watermark():
    """高风险文本水印示例"""
    print("\n" + "=" * 60)
    print("示例6: 高风险文本水印")
    print("=" * 60)
    
    info = WatermarkInfo(
        username="周八",
        employee_id="EMP006",
        department="安全部",
        auto_fill=True
    )
    
    # 使用高风险预设
    style = TextWatermarkStyle.high_risk_preset()
    
    content = "这是一份机密文档..."
    
    wm = TextWatermark()
    watermarked = wm.embed(content, info, style)
    
    print("高风险文本水印:")
    print(watermarked)
    print("✓ 高风险水印完成")


def example_html_watermark():
    """HTML水印示例"""
    print("\n" + "=" * 60)
    print("示例7: HTML水印")
    print("=" * 60)
    
    info = WatermarkInfo(
        username="吴九",
        employee_id="EMP007",
        auto_fill=True
    )
    
    html_content = """<!DOCTYPE html>
<html>
<head>
    <title>测试文档</title>
</head>
<body>
    <h1>标题</h1>
    <p>这是HTML文档内容</p>
</body>
</html>"""
    
    wm = HTMLWatermark()
    watermarked = wm.embed(html_content, info)
    
    print("HTML水印（固定定位覆盖层）:")
    print(watermarked[:600] + "..." if len(watermarked) > 600 else watermarked)
    print("✓ HTML水印完成")


def example_markdown_watermark():
    """Markdown水印示例"""
    print("\n" + "=" * 60)
    print("示例8: Markdown水印")
    print("=" * 60)
    
    info = WatermarkInfo(
        username="郑十",
        employee_id="EMP008",
        auto_fill=True
    )
    
    md_content = """# 项目文档

## 概述
这是项目概述部分。

## 功能列表
- 功能1
- 功能2
- 功能3"""
    
    wm = MarkdownWatermark()
    watermarked = wm.embed(md_content, info)
    
    print("Markdown水印:")
    print(watermarked)
    print("✓ Markdown水印完成")


def example_extract_watermark():
    """提取水印示例"""
    print("\n" + "=" * 60)
    print("示例9: 提取水印")
    print("=" * 60)
    
    info = WatermarkInfo(
        username="测试用户",
        employee_id="TEST001",
        auto_fill=True
    )
    
    content = "这是测试文本内容"
    
    wm = TextWatermark()
    
    # 添加水印
    watermarked = wm.embed(content, info)
    
    # 提取水印
    extracted = wm.extract(watermarked)
    
    if extracted:
        print("提取的水印信息:")
        print(f"  用户名: {extracted.username}")
        print(f"  工号: {extracted.employee_id}")
        print(f"  时间戳: {extracted.timestamp}")
    else:
        print("未能提取水印")
    
    print("✓ 提取水印完成")


def example_verify_watermark():
    """验证水印示例"""
    print("\n" + "=" * 60)
    print("示例10: 验证水印完整性")
    print("=" * 60)
    
    info = WatermarkInfo(
        username="验证测试",
        employee_id="VERIFY001",
        auto_fill=True
    )
    
    content = "这是需要验证的文本"
    
    wm = TextWatermark()
    watermarked = wm.embed(content, info)
    
    # 验证水印
    result = wm.verify_watermark(watermarked, info)
    
    print("验证结果:")
    print(f"  有效性: {'✓ 有效' if result['is_valid'] else '✗ 无效'}")
    print(f"  找到水印: {'✓ 是' if result['watermark_found'] else '✗ 否'}")
    print(f"  校验码有效: {'✓ 是' if result['checksum_valid'] else '✗ 否'}")
    print(f"  信息匹配: {'✓ 是' if result['info_match'] else '✗ 否'}")
    
    if result['details']:
        print(f"  详情: {', '.join(result['details'])}")
    
    print("✓ 验证水印完成")


def example_file_watermark():
    """文件水印示例"""
    print("\n" + "=" * 60)
    print("示例11: 文件水印操作")
    print("=" * 60)
    
    info = WatermarkInfo(
        username="文件测试",
        employee_id="FILE001",
        auto_fill=True
    )
    
    wm = TextWatermark()
    
    # 添加水印到文件
    # wm.embed_to_file("input.txt", "output.txt", info)
    
    # 从文件提取水印
    # extracted = wm.extract_from_file("output.txt")
    
    print("文件操作方法:")
    print("  - embed_to_file(input, output, info, style)")
    print("  - extract_from_file(file_path)")
    print("  - batch_embed(input_dir, output_dir, info)")
    print("✓ 文件水印演示完成")


if __name__ == "__main__":
    print("VisualWM 文本水印示例")
    print("=" * 60)
    
    example_basic_text_watermark()
    example_header_watermark()
    example_footer_watermark()
    example_inline_watermark()
    example_margin_watermark()
    example_high_risk_text_watermark()
    example_html_watermark()
    example_markdown_watermark()
    example_extract_watermark()
    example_verify_watermark()
    example_file_watermark()
    
    print("\n" + "=" * 60)
    print("所有文本水印示例演示完成!")
    print("=" * 60)



"""
图片水印处理器 - 支持图片添加明水印和提取
"""

import os
import math
from pathlib import Path
from typing import Optional, Tuple, Union, List
from PIL import Image, ImageDraw, ImageFont, ImageEnhance, ImageFilter
import numpy as np

from visualwm.core.info import WatermarkInfo
from visualwm.core.style import WatermarkStyle, WatermarkPosition


class ImageWatermark:
    """
    图片水印处理器
    
    支持在图片上添加明水印，并从图片中提取水印信息。
    
    Features:
        - 多种位置模式：平铺、居中、角落
        - 自定义样式：字体、颜色、透明度、旋转
        - 支持多种图片格式
        - 水印信息提取（OCR）
    """
    
    # 支持的图片格式
    SUPPORTED_FORMATS = {'.png', '.jpg', '.jpeg', '.bmp', '.gif', '.tiff', '.webp'}
    
    # 默认字体搜索路径
    FONT_PATHS = [
        os.path.join(os.path.dirname(__file__), "..", "font"),  # SDK内置字体目录
        "/usr/share/fonts/opentype/noto/",
        "/usr/share/fonts/truetype/wqy/",
        "/usr/share/fonts/truetype/dejavu/",
        "/usr/share/fonts/truetype/",
        "/usr/share/fonts/opentype/",
        "/usr/share/fonts/",
        "/System/Library/Fonts/",
        "/Library/Fonts/",
        "C:/Windows/Fonts/",
        "./fonts/",
    ]
    
    # 中文字体文件名映射
    CJK_FONT_FILES = [
        "NotoSerifSC-VariableFont_wght.ttf",  # SDK内置字体
        "NotoSansCJK-Regular.ttc",
        "NotoSansCJKsc-Regular.otf", 
        "wqy-zenhei.ttc",
        "wqy-microhei.ttc",
        "DejaVuSans.ttf",
    ]
    
    def __init__(self, default_style: Optional[WatermarkStyle] = None):
        """
        初始化图片水印处理器
        
        Args:
            default_style: 默认水印样式
        """
        self.default_style = default_style or WatermarkStyle.default()
        self._font_cache = {}
    
    def embed(
        self,
        input_path: Union[str, Path],
        output_path: Union[str, Path],
        info: WatermarkInfo,
        style: Optional[WatermarkStyle] = None,
    ) -> str:
        """
        在图片中嵌入水印
        
        Args:
            input_path: 输入图片路径
            output_path: 输出图片路径
            info: 水印信息
            style: 水印样式（可选，使用默认样式）
            
        Returns:
            输出文件路径
        """
        input_path = Path(input_path)
        output_path = Path(output_path)
        style = style or self.default_style
        
        # 验证输入文件
        if not input_path.exists():
            raise FileNotFoundError(f"输入文件不存在: {input_path}")
        
        if input_path.suffix.lower() not in self.SUPPORTED_FORMATS:
            raise ValueError(f"不支持的图片格式: {input_path.suffix}")
        
        # 打开图片
        image = Image.open(input_path)
        
        # 确保是RGBA模式以支持透明度
        if image.mode != 'RGBA':
            image = image.convert('RGBA')
        
        # 添加水印
        watermarked = self._add_watermark(image, info, style)
        
        # 保存图片
        output_path.parent.mkdir(parents=True, exist_ok=True)
        
        # 根据输出格式处理
        if output_path.suffix.lower() in {'.jpg', '.jpeg'}:
            # JPEG不支持透明度，转换为RGB
            watermarked = watermarked.convert('RGB')
        
        watermarked.save(output_path, quality=95)
        
        return str(output_path)
    
    def embed_to_bytes(
        self,
        image_bytes: bytes,
        info: WatermarkInfo,
        style: Optional[WatermarkStyle] = None,
        format: str = "PNG"
    ) -> bytes:
        """
        在内存中处理图片并返回字节
        
        Args:
            image_bytes: 输入图片字节
            info: 水印信息
            style: 水印样式
            format: 输出格式
            
        Returns:
            带水印的图片字节
        """
        from io import BytesIO
        
        style = style or self.default_style
        
        # 从字节加载图片
        image = Image.open(BytesIO(image_bytes))
        if image.mode != 'RGBA':
            image = image.convert('RGBA')
        
        # 添加水印
        watermarked = self._add_watermark(image, info, style)
        
        # 输出到字节
        output = BytesIO()
        if format.upper() in {'JPG', 'JPEG'}:
            watermarked = watermarked.convert('RGB')
        watermarked.save(output, format=format, quality=95)
        
        return output.getvalue()
    
    def _add_watermark(
        self,
        image: Image.Image,
        info: WatermarkInfo,
        style: WatermarkStyle
    ) -> Image.Image:
        """
        添加水印到图片
        
        Args:
            image: PIL图片对象
            info: 水印信息
            style: 水印样式
            
        Returns:
            带水印的图片
        """
        # 获取水印文本
        watermark_text = self._format_watermark_text(info, style)
        
        # 根据位置模式添加水印
        position = style.position.lower()
        
        if position == "tile":
            return self._add_tiled_watermark(image, watermark_text, style)
        elif position == "center":
            return self._add_centered_watermark(image, watermark_text, style)
        else:
            return self._add_positioned_watermark(image, watermark_text, style, position)
    
    def _format_watermark_text(self, info: WatermarkInfo, style: WatermarkStyle) -> str:
        """格式化水印文本"""
        text = info.to_string(separator=" | ", compact=True)
        
        if style.prefix:
            text = f"{style.prefix} {text}"
        if style.suffix:
            text = f"{text} {style.suffix}"
        
        return text
    
    def _get_font(self, style: WatermarkStyle) -> ImageFont.FreeTypeFont:
        """获取字体对象"""
        cache_key = (style.font_name, style.font_size, style.bold, style.italic)
        
        if cache_key in self._font_cache:
            return self._font_cache[cache_key]
        
        font = None
        
        # 首先尝试直接加载CJK字体文件（优先级最高）
        for font_file in self.CJK_FONT_FILES:
            for font_path in self.FONT_PATHS:
                full_path = os.path.join(font_path, font_file)
                if os.path.exists(full_path):
                    try:
                        font = ImageFont.truetype(full_path, style.font_size)
                        self._font_cache[cache_key] = font
                        return font
                    except (IOError, OSError):
                        continue
        
        # 尝试加载指定字体
        font_names = [style.font_name]
        
        # 添加备选字体
        if style.bold:
            font_names.append(f"{style.font_name}-Bold")
            font_names.append(f"{style.font_name}Bold")
        
        # 添加中文字体备选
        font_names.extend([
            "SimHei", "Microsoft YaHei", "STHeiti", "Noto Sans CJK SC",
            "WenQuanYi Micro Hei", "DejaVuSans", "Arial Unicode MS"
        ])
        
        for font_name in font_names:
            # 尝试直接加载
            try:
                font = ImageFont.truetype(font_name, style.font_size)
                break
            except (IOError, OSError):
                pass
            
            # 在字体目录中搜索
            for font_path in self.FONT_PATHS:
                for ext in ['.ttf', '.ttc', '.otf']:
                    try:
                        full_path = os.path.join(font_path, f"{font_name}{ext}")
                        if os.path.exists(full_path):
                            font = ImageFont.truetype(full_path, style.font_size)
                            break
                    except (IOError, OSError):
                        continue
                if font:
                    break
            
            if font:
                break
        
        # 如果所有字体都失败，使用默认字体
        if font is None:
            font = ImageFont.load_default()
        
        self._font_cache[cache_key] = font
        return font
    
    def _add_tiled_watermark(
        self,
        image: Image.Image,
        text: str,
        style: WatermarkStyle
    ) -> Image.Image:
        """添加平铺水印"""
        width, height = image.size
        font = self._get_font(style)
        
        # 创建水印层
        watermark_layer = Image.new('RGBA', (width, height), (0, 0, 0, 0))
        draw = ImageDraw.Draw(watermark_layer)
        
        # 获取文本尺寸
        bbox = draw.textbbox((0, 0), text, font=font)
        text_width = bbox[2] - bbox[0]
        text_height = bbox[3] - bbox[1]
        
        # 创建单个水印图像（带旋转）
        diagonal = int(math.sqrt(text_width**2 + text_height**2)) + 20
        single_watermark = Image.new('RGBA', (diagonal, diagonal), (0, 0, 0, 0))
        single_draw = ImageDraw.Draw(single_watermark)
        
        # 计算居中位置
        x = (diagonal - text_width) // 2
        y = (diagonal - text_height) // 2
        
        # 绘制阴影
        if style.shadow:
            shadow_x = x + style.shadow_offset[0]
            shadow_y = y + style.shadow_offset[1]
            shadow_color = (*style.shadow_color, int(style.opacity * 128))
            single_draw.text((shadow_x, shadow_y), text, font=font, fill=shadow_color)
        
        # 绘制文本
        single_draw.text((x, y), text, font=font, fill=style.rgba_color)
        
        # 旋转水印
        single_watermark = single_watermark.rotate(
            style.rotation, 
            expand=False, 
            resample=Image.BICUBIC
        )
        
        # 平铺水印
        spacing_x = text_width + style.column_spacing
        spacing_y = text_height + style.line_spacing
        
        for y_pos in range(-diagonal, height + diagonal, spacing_y):
            # 奇偶行错开
            offset = spacing_x // 2 if (y_pos // spacing_y) % 2 else 0
            for x_pos in range(-diagonal + offset, width + diagonal, spacing_x):
                watermark_layer.paste(single_watermark, (x_pos, y_pos), single_watermark)
        
        # 合并图层
        return Image.alpha_composite(image, watermark_layer)
    
    def _add_centered_watermark(
        self,
        image: Image.Image,
        text: str,
        style: WatermarkStyle
    ) -> Image.Image:
        """添加居中水印"""
        width, height = image.size
        font = self._get_font(style)
        
        # 创建水印层
        watermark_layer = Image.new('RGBA', (width, height), (0, 0, 0, 0))
        draw = ImageDraw.Draw(watermark_layer)
        
        # 获取文本尺寸
        bbox = draw.textbbox((0, 0), text, font=font)
        text_width = bbox[2] - bbox[0]
        text_height = bbox[3] - bbox[1]
        
        # 创建单个水印图像
        padding = 40
        single_watermark = Image.new(
            'RGBA', 
            (text_width + padding * 2, text_height + padding * 2), 
            (0, 0, 0, 0)
        )
        single_draw = ImageDraw.Draw(single_watermark)
        
        # 绘制背景
        if style.background_color:
            single_draw.rectangle(
                [0, 0, text_width + padding * 2, text_height + padding * 2],
                fill=style.background_color
            )
        
        # 绘制边框
        if style.border:
            single_draw.rectangle(
                [0, 0, text_width + padding * 2 - 1, text_height + padding * 2 - 1],
                outline=(*style.border_color, int(style.opacity * 255)),
                width=style.border_width
            )
        
        # 绘制阴影
        if style.shadow:
            shadow_color = (*style.shadow_color, int(style.opacity * 128))
            single_draw.text(
                (padding + style.shadow_offset[0], padding + style.shadow_offset[1]),
                text, font=font, fill=shadow_color
            )
        
        # 绘制文本
        single_draw.text((padding, padding), text, font=font, fill=style.rgba_color)
        
        # 旋转水印
        single_watermark = single_watermark.rotate(
            style.rotation,
            expand=True,
            resample=Image.BICUBIC
        )
        
        # 计算居中位置
        wm_width, wm_height = single_watermark.size
        x = (width - wm_width) // 2
        y = (height - wm_height) // 2
        
        # 粘贴水印
        watermark_layer.paste(single_watermark, (x, y), single_watermark)
        
        return Image.alpha_composite(image, watermark_layer)
    
    def _add_positioned_watermark(
        self,
        image: Image.Image,
        text: str,
        style: WatermarkStyle,
        position: str
    ) -> Image.Image:
        """添加定位水印（角落等）"""
        width, height = image.size
        font = self._get_font(style)
        
        # 创建水印层
        watermark_layer = Image.new('RGBA', (width, height), (0, 0, 0, 0))
        draw = ImageDraw.Draw(watermark_layer)
        
        # 获取文本尺寸
        bbox = draw.textbbox((0, 0), text, font=font)
        text_width = bbox[2] - bbox[0]
        text_height = bbox[3] - bbox[1]
        
        # 计算位置
        margin = style.margin
        
        position_map = {
            "top_left": (margin, margin),
            "top_right": (width - text_width - margin, margin),
            "bottom_left": (margin, height - text_height - margin),
            "bottom_right": (width - text_width - margin, height - text_height - margin),
            "top_center": ((width - text_width) // 2, margin),
            "bottom_center": ((width - text_width) // 2, height - text_height - margin),
        }
        
        x, y = position_map.get(position, (margin, margin))
        
        # 绘制背景
        if style.background_color:
            padding = 5
            draw.rectangle(
                [x - padding, y - padding, x + text_width + padding, y + text_height + padding],
                fill=style.background_color
            )
        
        # 绘制边框
        if style.border:
            padding = 5
            draw.rectangle(
                [x - padding, y - padding, x + text_width + padding, y + text_height + padding],
                outline=(*style.border_color, int(style.opacity * 255)),
                width=style.border_width
            )
        
        # 绘制阴影
        if style.shadow:
            shadow_color = (*style.shadow_color, int(style.opacity * 128))
            draw.text(
                (x + style.shadow_offset[0], y + style.shadow_offset[1]),
                text, font=font, fill=shadow_color
            )
        
        # 绘制文本
        draw.text((x, y), text, font=font, fill=style.rgba_color)
        
        return Image.alpha_composite(image, watermark_layer)
    
    def extract(
        self,
        image_path: Union[str, Path],
        use_ocr: bool = False
    ) -> Optional[WatermarkInfo]:
        """
        从图片中提取水印信息
        
        Args:
            image_path: 图片路径
            use_ocr: 是否使用OCR（需要额外依赖）
            
        Returns:
            提取的水印信息，如果无法提取返回None
            
        Note:
            明水印的提取依赖OCR技术，需要安装pytesseract等OCR库
        """
        if use_ocr:
            return self._extract_with_ocr(image_path)
        else:
            # 不使用OCR时，返回提示信息
            print("警告：明水印提取需要启用OCR功能，请设置 use_ocr=True")
            print("需要安装：pip install pytesseract")
            return None
    
    def _extract_with_ocr(self, image_path: Union[str, Path]) -> Optional[WatermarkInfo]:
        """使用OCR提取水印"""
        try:
            import pytesseract
        except ImportError:
            raise ImportError(
                "OCR功能需要安装pytesseract: pip install pytesseract\n"
                "同时需要安装Tesseract OCR引擎"
            )
        
        image = Image.open(image_path)
        
        # 图像预处理以提高OCR准确率
        # 转换为灰度
        gray = image.convert('L')
        
        # 增强对比度
        enhancer = ImageEnhance.Contrast(gray)
        enhanced = enhancer.enhance(2.0)
        
        # OCR识别
        text = pytesseract.image_to_string(enhanced, lang='chi_sim+eng')
        
        # 尝试解析水印信息
        if text.strip():
            try:
                return WatermarkInfo.from_string(text.strip())
            except Exception:
                # 解析失败，返回原始文本
                return WatermarkInfo(custom_data={"raw_text": text.strip()})
        
        return None
    
    def batch_embed(
        self,
        input_dir: Union[str, Path],
        output_dir: Union[str, Path],
        info: WatermarkInfo,
        style: Optional[WatermarkStyle] = None,
        recursive: bool = False
    ) -> List[str]:
        """
        批量添加水印
        
        Args:
            input_dir: 输入目录
            output_dir: 输出目录
            info: 水印信息
            style: 水印样式
            recursive: 是否递归处理子目录
            
        Returns:
            处理的文件列表
        """
        input_dir = Path(input_dir)
        output_dir = Path(output_dir)
        
        if not input_dir.is_dir():
            raise ValueError(f"输入路径不是目录: {input_dir}")
        
        processed = []
        
        # 获取文件列表
        if recursive:
            files = input_dir.rglob("*")
        else:
            files = input_dir.glob("*")
        
        for input_file in files:
            if input_file.suffix.lower() in self.SUPPORTED_FORMATS:
                # 计算输出路径
                relative_path = input_file.relative_to(input_dir)
                output_file = output_dir / relative_path
                
                try:
                    self.embed(input_file, output_file, info, style)
                    processed.append(str(output_file))
                except Exception as e:
                    print(f"处理失败 {input_file}: {e}")
        
        return processed
    
    def add_multiline_watermark(
        self,
        input_path: Union[str, Path],
        output_path: Union[str, Path],
        info: WatermarkInfo,
        style: Optional[WatermarkStyle] = None,
    ) -> str:
        """
        添加多行水印
        
        Args:
            input_path: 输入图片路径
            output_path: 输出图片路径
            info: 水印信息
            style: 水印样式
            
        Returns:
            输出文件路径
        """
        input_path = Path(input_path)
        output_path = Path(output_path)
        style = style or self.default_style
        
        image = Image.open(input_path)
        if image.mode != 'RGBA':
            image = image.convert('RGBA')
        
        width, height = image.size
        font = self._get_font(style)
        
        # 创建水印层
        watermark_layer = Image.new('RGBA', (width, height), (0, 0, 0, 0))
        draw = ImageDraw.Draw(watermark_layer)
        
        # 获取多行文本
        lines = info.to_multiline_string().split('\n')
        
        # 计算总高度
        line_heights = []
        total_height = 0
        for line in lines:
            bbox = draw.textbbox((0, 0), line, font=font)
            h = bbox[3] - bbox[1]
            line_heights.append(h)
            total_height += h + 5  # 5px行间距
        
        # 计算起始位置（居中）
        start_y = (height - total_height) // 2
        
        # 绘制每行
        y = start_y
        for i, line in enumerate(lines):
            bbox = draw.textbbox((0, 0), line, font=font)
            text_width = bbox[2] - bbox[0]
            x = (width - text_width) // 2
            
            draw.text((x, y), line, font=font, fill=style.rgba_color)
            y += line_heights[i] + 5
        
        # 合并图层
        result = Image.alpha_composite(image, watermark_layer)
        
        # 保存
        output_path.parent.mkdir(parents=True, exist_ok=True)
        if output_path.suffix.lower() in {'.jpg', '.jpeg'}:
            result = result.convert('RGB')
        result.save(output_path, quality=95)
        
        return str(output_path)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
VisualWM 示例代码 - 视频水印
"""

from visualwm import VideoWatermark, WatermarkInfo, WatermarkStyle, RiskLevel


def example_basic_video_watermark():
    """基本视频水印示例"""
    print("=" * 60)
    print("示例1: 基本视频水印")
    print("=" * 60)
    
    # 创建水印信息
    info = WatermarkInfo(
        username="张三",
        employee_id="EMP001",
        device_id="DEV-2024-001",
        auto_fill=True
    )
    
    print(f"水印信息: {info}")
    
    # 创建视频水印处理器
    wm = VideoWatermark()
    
    # 定义进度回调
    def progress_callback(current, total):
        percent = (current / total) * 100
        print(f"\r处理进度: {current}/{total} ({percent:.1f}%)", end="")
    
    # 添加水印
    # wm.embed(
    #     "input.mp4",
    #     "output.mp4",
    #     info,
    #     progress_callback=progress_callback
    # )
    
    print("\n✓ 基本视频水印配置完成")


def example_dynamic_timestamp_watermark():
    """动态时间戳水印示例"""
    print("\n" + "=" * 60)
    print("示例2: 动态时间戳水印（每帧更新）")
    print("=" * 60)
    
    info = WatermarkInfo(
        username="李四",
        employee_id="EMP002",
        auto_fill=True
    )
    
    style = WatermarkStyle(
        font_size=18,
        opacity=0.4,
        position="bottom_right",
        margin=20
    )
    
    # wm = VideoWatermark()
    # wm.embed(
    #     "input.mp4",
    #     "output_dynamic.mp4",
    #     info,
    #     style,
    #     dynamic_timestamp=True  # 每帧更新时间戳
    # )
    
    print("动态时间戳功能: 每帧显示实时时间")
    print("✓ 动态时间戳配置完成")


def example_tiled_video_watermark():
    """平铺视频水印示例"""
    print("\n" + "=" * 60)
    print("示例3: 平铺视频水印")
    print("=" * 60)
    
    info = WatermarkInfo(
        username="王五",
        employee_id="EMP003",
        department="研发部",
        auto_fill=True
    )
    
    # 平铺样式
    style = WatermarkStyle(
        font_size=20,
        color=(150, 150, 150),
        opacity=0.3,
        rotation=-30,
        position="tile",
        line_spacing=100,
        column_spacing=200
    )
    
    print(f"水印样式: 平铺模式")
    print(f"旋转角度: {style.rotation}°")
    print(f"行间距: {style.line_spacing}px")
    print(f"列间距: {style.column_spacing}px")
    
    # wm = VideoWatermark()
    # wm.embed("input.mp4", "output_tiled.mp4", info, style)
    
    print("✓ 平铺水印配置完成")


def example_high_risk_video_watermark():
    """高风险视频水印示例"""
    print("\n" + "=" * 60)
    print("示例4: 高风险（机密）视频水印")
    print("=" * 60)
    
    info = WatermarkInfo(
        username="赵六",
        employee_id="EMP004",
        department="安全部",
        auto_fill=True
    )
    
    # 使用高风险预设
    style = WatermarkStyle.critical_risk_preset()
    
    print(f"风险等级: {style.risk_level}")
    print(f"前缀: {style.prefix}")
    print(f"颜色: {style.hex_color} (红色)")
    print(f"透明度: {style.opacity}")
    
    # wm = VideoWatermark()
    # wm.embed("secret_video.mp4", "output_secret.mp4", info, style)
    
    print("✓ 高风险视频水印配置完成")


def example_get_video_info():
    """获取视频信息示例"""
    print("\n" + "=" * 60)
    print("示例5: 获取视频信息")
    print("=" * 60)
    
    wm = VideoWatermark()
    
    # video_info = wm.get_video_info("input.mp4")
    # print(f"分辨率: {video_info['width']}x{video_info['height']}")
    # print(f"帧率: {video_info['fps']} FPS")
    # print(f"总帧数: {video_info['total_frames']}")
    # print(f"时长: {video_info['duration']:.2f} 秒")
    
    print("视频信息获取功能演示")
    print("✓ 获取视频信息完成")


def example_extract_video_frame():
    """提取视频帧示例"""
    print("\n" + "=" * 60)
    print("示例6: 提取视频帧")
    print("=" * 60)
    
    wm = VideoWatermark()
    
    # 提取第100帧
    # frame = wm.extract_frame("watermarked_video.mp4", frame_number=100)
    # frame.save("extracted_frame.png")
    
    # 从帧中提取水印
    # info = wm.extract("watermarked_video.mp4", frame_number=100, use_ocr=True)
    
    print("可提取任意帧进行分析")
    print("✓ 提取视频帧完成")


def example_batch_video_processing():
    """批量视频处理示例"""
    print("\n" + "=" * 60)
    print("示例7: 批量视频处理")
    print("=" * 60)
    
    info = WatermarkInfo(
        username="批量处理用户",
        employee_id="BATCH001",
        auto_fill=True
    )
    
    def batch_progress(filename, current, total):
        print(f"处理中 ({current}/{total}): {filename}")
    
    # wm = VideoWatermark()
    # processed = wm.batch_embed(
    #     input_dir="./input_videos/",
    #     output_dir="./output_videos/",
    #     info=info,
    #     recursive=True,
    #     progress_callback=batch_progress
    # )
    # print(f"已处理 {len(processed)} 个视频文件")
    
    print("✓ 批量视频处理配置完成")


if __name__ == "__main__":
    print("VisualWM 视频水印示例")
    print("=" * 60)
    
    example_basic_video_watermark()
    example_dynamic_timestamp_watermark()
    example_tiled_video_watermark()
    example_high_risk_video_watermark()
    example_get_video_info()
    example_extract_video_frame()
    example_batch_video_processing()
    
    print("\n" + "=" * 60)
    print("所有视频水印示例演示完成!")
    print("=" * 60)



"""
视频水印处理器 - 支持视频添加明水印和提取

支持多核并行处理加速，保留原始音频
"""

import os
import shutil
import subprocess
import tempfile
import multiprocessing as mp
from multiprocessing import shared_memory
from concurrent.futures import ProcessPoolExecutor, ThreadPoolExecutor, as_completed
from pathlib import Path
from typing import Optional, Union, List, Callable, Tuple
import cv2
import numpy as np
from PIL import Image, ImageDraw, ImageFont

from visualwm.core.info import WatermarkInfo
from visualwm.core.style import WatermarkStyle
from visualwm.image.watermark import ImageWatermark


# 全局变量用于进程间共享水印叠加层
_global_watermark_overlay = None
_global_style_opacity = None
_global_frame_shape = None


def _init_worker(watermark_overlay: np.ndarray, opacity: float):
    """初始化worker进程的全局变量"""
    global _global_watermark_overlay, _global_style_opacity, _global_frame_shape
    _global_watermark_overlay = watermark_overlay
    _global_style_opacity = opacity
    _global_frame_shape = watermark_overlay.shape[:2]


def _process_single_frame(frame: np.ndarray) -> np.ndarray:
    """处理单帧（在子进程中运行）"""
    global _global_watermark_overlay, _global_style_opacity
    
    # 转换帧为RGBA
    frame_rgba = cv2.cvtColor(frame, cv2.COLOR_BGR2RGBA)
    
    # 获取水印的alpha通道
    alpha = _global_watermark_overlay[:, :, 3:4] / 255.0
    
    # 混合
    blended = frame_rgba.astype(np.float32)
    watermark_rgb = _global_watermark_overlay[:, :, :3].astype(np.float32)
    
    # 只在有水印的地方混合
    mask = alpha > 0
    blended[:, :, :3] = np.where(
        np.broadcast_to(mask, blended[:, :, :3].shape),
        frame_rgba[:, :, :3] * (1 - alpha) + watermark_rgb * alpha,
        frame_rgba[:, :, :3]
    )
    
    # 转回BGR
    result = cv2.cvtColor(blended.astype(np.uint8), cv2.COLOR_RGBA2BGR)
    return result


def _process_frame_batch(args: Tuple[int, List[np.ndarray]]) -> Tuple[int, List[np.ndarray]]:
    """
    处理一批视频帧（在子进程中运行）
    
    Args:
        args: (batch_index, frames) 批次索引和帧列表
        
    Returns:
        (batch_index, processed_frames) 批次索引和处理后的帧列表
    """
    batch_idx, frames = args
    processed = [_process_single_frame(frame) for frame in frames]
    return batch_idx, processed


class VideoWatermark:
    """
    视频水印处理器
    
    支持在视频上添加明水印，并从视频中提取水印信息。
    
    Features:
        - 支持多种视频格式（MP4、AVI、MOV等）
        - 逐帧添加水印
        - 动态水印（时间戳实时更新）
        - 批量处理
        - 进度回调
    """
    
    # 支持的视频格式
    SUPPORTED_FORMATS = {'.mp4', '.avi', '.mov', '.mkv', '.wmv', '.flv', '.webm'}
    
    # 常用视频编解码器
    CODECS = {
        '.mp4': 'mp4v',
        '.avi': 'XVID',
        '.mov': 'mp4v',
        '.mkv': 'XVID',
        '.wmv': 'WMV2',
        '.webm': 'VP80',
    }
    
    def __init__(self, default_style: Optional[WatermarkStyle] = None):
        """
        初始化视频水印处理器
        
        Args:
            default_style: 默认水印样式
        """
        self.default_style = default_style or WatermarkStyle.default()
        self._image_watermark = ImageWatermark(default_style)
        # 默认使用CPU核心数的一半（避免过度并行）
        self._num_workers = max(1, mp.cpu_count() // 2)
        # 检测ffmpeg是否可用
        self._ffmpeg_available = self._check_ffmpeg()
    
    def _check_ffmpeg(self) -> bool:
        """检查ffmpeg是否可用"""
        try:
            result = subprocess.run(
                ['ffmpeg', '-version'],
                capture_output=True,
                text=True
            )
            return result.returncode == 0
        except FileNotFoundError:
            return False
    
    def _has_audio(self, video_path: Union[str, Path]) -> bool:
        """检查视频是否包含音频轨道"""
        if not self._ffmpeg_available:
            return False
        try:
            result = subprocess.run(
                ['ffprobe', '-v', 'error', '-select_streams', 'a',
                 '-show_entries', 'stream=codec_type', '-of', 'csv=p=0',
                 str(video_path)],
                capture_output=True,
                text=True
            )
            return 'audio' in result.stdout
        except Exception:
            return False
    
    def _extract_audio(self, video_path: Union[str, Path], audio_path: Union[str, Path]) -> bool:
        """从视频中提取音频"""
        if not self._ffmpeg_available:
            return False
        try:
            result = subprocess.run(
                ['ffmpeg', '-y', '-i', str(video_path), '-vn', '-acodec', 'copy', str(audio_path)],
                capture_output=True,
                text=True
            )
            return result.returncode == 0 and Path(audio_path).exists()
        except Exception:
            return False
    
    def _merge_audio_video(
        self, 
        video_path: Union[str, Path], 
        audio_path: Union[str, Path], 
        output_path: Union[str, Path]
    ) -> bool:
        """合并音频和视频"""
        if not self._ffmpeg_available:
            return False
        try:
            result = subprocess.run(
                ['ffmpeg', '-y', '-i', str(video_path), '-i', str(audio_path),
                 '-c:v', 'copy', '-c:a', 'aac', '-strict', 'experimental',
                 str(output_path)],
                capture_output=True,
                text=True
            )
            return result.returncode == 0
        except Exception:
            return False
    
    def set_workers(self, num_workers: int):
        """
        设置并行处理的worker数量
        
        Args:
            num_workers: worker数量，默认为CPU核心数的一半
        """
        self._num_workers = max(1, num_workers)
    
    def embed(
        self,
        input_path: Union[str, Path],
        output_path: Union[str, Path],
        info: WatermarkInfo,
        style: Optional[WatermarkStyle] = None,
        dynamic_timestamp: bool = False,
        progress_callback: Optional[Callable[[int, int], None]] = None,
        parallel: bool = True,
        num_workers: Optional[int] = None,
        batch_size: int = 50,
        mode: str = "thread",
    ) -> str:
        """
        在视频中嵌入水印
        
        Args:
            input_path: 输入视频路径
            output_path: 输出视频路径
            info: 水印信息
            style: 水印样式
            dynamic_timestamp: 是否使用动态时间戳（每帧更新）
            progress_callback: 进度回调函数 (current_frame, total_frames)
            parallel: 是否启用多核并行处理（默认True）
            num_workers: 并行worker数量（默认为CPU核心数的一半）
            batch_size: 每批处理的帧数（仅用于process模式，默认50帧）
            mode: 并行模式 "thread"（多线程，默认，推荐）或 "process"（多进程）
            
        Returns:
            输出文件路径
        """
        input_path = Path(input_path)
        output_path = Path(output_path)
        style = style or self.default_style
        
        # 验证输入文件
        if not input_path.exists():
            raise FileNotFoundError(f"输入文件不存在: {input_path}")
        
        if input_path.suffix.lower() not in self.SUPPORTED_FORMATS:
            raise ValueError(f"不支持的视频格式: {input_path.suffix}")
        
        # 动态时间戳模式不支持并行（因为每帧时间戳不同）
        if dynamic_timestamp:
            parallel = False
        
        # 检查是否有音频需要保留
        has_audio = self._has_audio(input_path)
        
        # 如果有音频，先提取
        audio_temp = None
        if has_audio:
            audio_temp = tempfile.NamedTemporaryFile(suffix='.aac', delete=False)
            audio_temp.close()
            if not self._extract_audio(input_path, audio_temp.name):
                has_audio = False
                os.unlink(audio_temp.name)
                audio_temp = None
        
        # 处理视频（不带音频）
        if has_audio:
            # 先输出到临时文件
            video_temp = tempfile.NamedTemporaryFile(suffix=output_path.suffix, delete=False)
            video_temp.close()
            temp_output = Path(video_temp.name)
        else:
            temp_output = output_path
        
        try:
            if parallel:
                if mode == "thread":
                    self._embed_parallel_thread(
                        input_path, temp_output, info, style,
                        progress_callback, num_workers
                    )
                else:
                    self._embed_parallel(
                        input_path, temp_output, info, style,
                        progress_callback, num_workers, batch_size
                    )
            else:
                self._embed_sequential(
                    input_path, temp_output, info, style,
                    dynamic_timestamp, progress_callback
                )
            
            # 如果有音频，合并音频和视频
            if has_audio and audio_temp:
                output_path.parent.mkdir(parents=True, exist_ok=True)
                if self._merge_audio_video(temp_output, audio_temp.name, output_path):
                    # 合并成功，删除临时文件
                    os.unlink(temp_output)
                else:
                    # 合并失败，使用无音频版本
                    shutil.move(str(temp_output), str(output_path))
        finally:
            # 清理音频临时文件
            if audio_temp and os.path.exists(audio_temp.name):
                os.unlink(audio_temp.name)
        
        return str(output_path)
    
    def _embed_parallel_thread(
        self,
        input_path: Path,
        output_path: Path,
        info: WatermarkInfo,
        style: WatermarkStyle,
        progress_callback: Optional[Callable[[int, int], None]] = None,
        num_workers: Optional[int] = None,
    ) -> str:
        """
        多线程并行处理视频水印（流式处理，内存友好）
        
        使用线程池进行并行处理，避免进程间数据拷贝开销
        """
        num_workers = num_workers or self._num_workers
        
        # 打开视频
        cap = cv2.VideoCapture(str(input_path))
        
        if not cap.isOpened():
            raise ValueError(f"无法打开视频文件: {input_path}")
        
        # 获取视频属性
        fps = cap.get(cv2.CAP_PROP_FPS)
        width = int(cap.get(cv2.CAP_PROP_FRAME_WIDTH))
        height = int(cap.get(cv2.CAP_PROP_FRAME_HEIGHT))
        total_frames = int(cap.get(cv2.CAP_PROP_FRAME_COUNT))
        
        # 创建输出目录
        output_path.parent.mkdir(parents=True, exist_ok=True)
        
        # 获取编解码器
        codec = self.CODECS.get(output_path.suffix.lower(), 'mp4v')
        fourcc = cv2.VideoWriter_fourcc(*codec)
        
        # 预生成水印叠加层
        watermark_overlay = self._create_watermark_overlay(width, height, info, style)
        
        # 预计算alpha通道
        alpha = watermark_overlay[:, :, 3:4].astype(np.float32) / 255.0
        watermark_rgb = watermark_overlay[:, :, :3].astype(np.float32)
        mask = alpha > 0
        
        def process_frame(frame):
            """处理单帧"""
            frame_rgba = cv2.cvtColor(frame, cv2.COLOR_BGR2RGBA)
            blended = frame_rgba.astype(np.float32)
            blended[:, :, :3] = np.where(
                np.broadcast_to(mask, blended[:, :, :3].shape),
                frame_rgba[:, :, :3] * (1 - alpha) + watermark_rgb * alpha,
                frame_rgba[:, :, :3]
            )
            return cv2.cvtColor(blended.astype(np.uint8), cv2.COLOR_RGBA2BGR)
        
        # 创建视频写入器
        out = cv2.VideoWriter(str(output_path), fourcc, fps, (width, height))
        
        try:
            frame_count = 0
            pending_results = {}
            next_write_idx = 0
            
            with ThreadPoolExecutor(max_workers=num_workers) as executor:
                futures = {}
                
                while True:
                    ret, frame = cap.read()
                    if not ret:
                        break
                    
                    # 提交帧处理任务
                    future = executor.submit(process_frame, frame.copy())
                    futures[future] = frame_count
                    frame_count += 1
                    
                    # 检查完成的任务并按顺序写入
                    done_futures = [f for f in futures if f.done()]
                    for future in done_futures:
                        idx = futures.pop(future)
                        pending_results[idx] = future.result()
                    
                    # 按顺序写入已完成的帧
                    while next_write_idx in pending_results:
                        out.write(pending_results.pop(next_write_idx))
                        next_write_idx += 1
                        if progress_callback:
                            progress_callback(next_write_idx, total_frames)
                
                # 等待剩余任务完成
                for future in as_completed(futures):
                    idx = futures[future]
                    pending_results[idx] = future.result()
                
                # 写入剩余帧
                while next_write_idx in pending_results:
                    out.write(pending_results.pop(next_write_idx))
                    next_write_idx += 1
                    if progress_callback:
                        progress_callback(next_write_idx, total_frames)
        
        finally:
            cap.release()
            out.release()
        
        return str(output_path)
    
    def _embed_parallel(
        self,
        input_path: Path,
        output_path: Path,
        info: WatermarkInfo,
        style: WatermarkStyle,
        progress_callback: Optional[Callable[[int, int], None]] = None,
        num_workers: Optional[int] = None,
        batch_size: int = 30,
    ) -> str:
        """
        多核并行处理视频水印
        
        使用多进程并行处理视频帧，显著提升处理速度
        """
        num_workers = num_workers or self._num_workers
        
        # 打开视频
        cap = cv2.VideoCapture(str(input_path))
        
        if not cap.isOpened():
            raise ValueError(f"无法打开视频文件: {input_path}")
        
        # 获取视频属性
        fps = cap.get(cv2.CAP_PROP_FPS)
        width = int(cap.get(cv2.CAP_PROP_FRAME_WIDTH))
        height = int(cap.get(cv2.CAP_PROP_FRAME_HEIGHT))
        total_frames = int(cap.get(cv2.CAP_PROP_FRAME_COUNT))
        
        # 创建输出目录
        output_path.parent.mkdir(parents=True, exist_ok=True)
        
        # 获取编解码器
        codec = self.CODECS.get(output_path.suffix.lower(), 'mp4v')
        fourcc = cv2.VideoWriter_fourcc(*codec)
        
        # 预生成水印叠加层
        watermark_overlay = self._create_watermark_overlay(width, height, info, style)
        
        # 读取所有帧
        all_frames = []
        while True:
            ret, frame = cap.read()
            if not ret:
                break
            all_frames.append(frame)
        cap.release()
        
        actual_total = len(all_frames)
        
        # 将帧分批
        batches = []
        for i in range(0, actual_total, batch_size):
            batch_frames = all_frames[i:i + batch_size]
            batches.append((len(batches), batch_frames))
        
        # 创建视频写入器
        out = cv2.VideoWriter(str(output_path), fourcc, fps, (width, height))
        
        try:
            # 多进程并行处理
            processed_batches = {}
            completed = 0
            
            with ProcessPoolExecutor(
                max_workers=num_workers,
                initializer=_init_worker,
                initargs=(watermark_overlay, style.opacity)
            ) as executor:
                # 提交所有批次
                futures = {
                    executor.submit(_process_frame_batch, batch): batch[0]
                    for batch in batches
                }
                
                # 收集结果
                for future in as_completed(futures):
                    batch_idx, processed_frames = future.result()
                    processed_batches[batch_idx] = processed_frames
                    completed += len(processed_frames)
                    
                    if progress_callback:
                        progress_callback(completed, actual_total)
            
            # 按顺序写入帧
            for batch_idx in range(len(batches)):
                for frame in processed_batches[batch_idx]:
                    out.write(frame)
        
        finally:
            out.release()
        
        return str(output_path)
    
    def _embed_sequential(
        self,
        input_path: Path,
        output_path: Path,
        info: WatermarkInfo,
        style: WatermarkStyle,
        dynamic_timestamp: bool = False,
        progress_callback: Optional[Callable[[int, int], None]] = None,
    ) -> str:
        """
        顺序处理视频水印（原始方法）
        """
        
        # 打开视频
        cap = cv2.VideoCapture(str(input_path))
        
        if not cap.isOpened():
            raise ValueError(f"无法打开视频文件: {input_path}")
        
        # 获取视频属性
        fps = cap.get(cv2.CAP_PROP_FPS)
        width = int(cap.get(cv2.CAP_PROP_FRAME_WIDTH))
        height = int(cap.get(cv2.CAP_PROP_FRAME_HEIGHT))
        total_frames = int(cap.get(cv2.CAP_PROP_FRAME_COUNT))
        
        # 创建输出目录
        output_path.parent.mkdir(parents=True, exist_ok=True)
        
        # 获取编解码器
        codec = self.CODECS.get(output_path.suffix.lower(), 'mp4v')
        fourcc = cv2.VideoWriter_fourcc(*codec)
        
        # 创建视频写入器
        out = cv2.VideoWriter(str(output_path), fourcc, fps, (width, height))
        
        try:
            # 预生成水印层（如果不是动态的）
            if not dynamic_timestamp:
                watermark_overlay = self._create_watermark_overlay(
                    width, height, info, style
                )
            
            frame_count = 0
            
            while True:
                ret, frame = cap.read()
                if not ret:
                    break
                
                # 动态时间戳模式：每帧更新时间
                if dynamic_timestamp:
                    info.refresh_dynamic_info()
                    watermark_overlay = self._create_watermark_overlay(
                        width, height, info, style
                    )
                
                # 应用水印
                watermarked_frame = self._apply_watermark_to_frame(
                    frame, watermark_overlay, style.opacity
                )
                
                # 写入帧
                out.write(watermarked_frame)
                
                frame_count += 1
                
                # 进度回调
                if progress_callback:
                    progress_callback(frame_count, total_frames)
        
        finally:
            cap.release()
            out.release()
        
        return str(output_path)
    
    def _create_watermark_overlay(
        self,
        width: int,
        height: int,
        info: WatermarkInfo,
        style: WatermarkStyle
    ) -> np.ndarray:
        """
        创建水印叠加层
        
        Args:
            width: 视频宽度
            height: 视频高度
            info: 水印信息
            style: 水印样式
            
        Returns:
            RGBA格式的numpy数组
        """
        # 创建透明图层
        overlay = Image.new('RGBA', (width, height), (0, 0, 0, 0))
        
        # 使用图片水印处理器的方法添加水印
        watermark_text = self._format_watermark_text(info, style)
        
        position = style.position.lower()
        
        if position == "tile":
            overlay = self._add_tiled_watermark_pil(overlay, watermark_text, style)
        elif position == "center":
            overlay = self._add_centered_watermark_pil(overlay, watermark_text, style)
        else:
            overlay = self._add_positioned_watermark_pil(overlay, watermark_text, style, position)
        
        # 转换为numpy数组
        return np.array(overlay)
    
    def _format_watermark_text(self, info: WatermarkInfo, style: WatermarkStyle) -> str:
        """格式化水印文本"""
        text = info.to_string(separator=" | ", compact=True)
        
        if style.prefix:
            text = f"{style.prefix} {text}"
        if style.suffix:
            text = f"{text} {style.suffix}"
        
        return text
    
    def _get_font(self, style: WatermarkStyle) -> ImageFont.FreeTypeFont:
        """获取字体"""
        return self._image_watermark._get_font(style)
    
    def _add_tiled_watermark_pil(
        self,
        image: Image.Image,
        text: str,
        style: WatermarkStyle
    ) -> Image.Image:
        """添加平铺水印（PIL版本）"""
        import math
        
        width, height = image.size
        font = self._get_font(style)
        draw = ImageDraw.Draw(image)
        
        # 获取文本尺寸
        bbox = draw.textbbox((0, 0), text, font=font)
        text_width = bbox[2] - bbox[0]
        text_height = bbox[3] - bbox[1]
        
        # 创建单个水印
        diagonal = int(math.sqrt(text_width**2 + text_height**2)) + 20
        single_watermark = Image.new('RGBA', (diagonal, diagonal), (0, 0, 0, 0))
        single_draw = ImageDraw.Draw(single_watermark)
        
        x = (diagonal - text_width) // 2
        y = (diagonal - text_height) // 2
        
        # 绘制阴影
        if style.shadow:
            shadow_color = (*style.shadow_color, int(style.opacity * 128))
            single_draw.text(
                (x + style.shadow_offset[0], y + style.shadow_offset[1]),
                text, font=font, fill=shadow_color
            )
        
        # 绘制文本
        single_draw.text((x, y), text, font=font, fill=style.rgba_color)
        
        # 旋转
        single_watermark = single_watermark.rotate(
            style.rotation, expand=False, resample=Image.BICUBIC
        )
        
        # 平铺
        spacing_x = text_width + style.column_spacing
        spacing_y = text_height + style.line_spacing
        
        for y_pos in range(-diagonal, height + diagonal, spacing_y):
            offset = spacing_x // 2 if (y_pos // spacing_y) % 2 else 0
            for x_pos in range(-diagonal + offset, width + diagonal, spacing_x):
                image.paste(single_watermark, (x_pos, y_pos), single_watermark)
        
        return image
    
    def _add_centered_watermark_pil(
        self,
        image: Image.Image,
        text: str,
        style: WatermarkStyle
    ) -> Image.Image:
        """添加居中水印"""
        width, height = image.size
        font = self._get_font(style)
        draw = ImageDraw.Draw(image)
        
        bbox = draw.textbbox((0, 0), text, font=font)
        text_width = bbox[2] - bbox[0]
        text_height = bbox[3] - bbox[1]
        
        x = (width - text_width) // 2
        y = (height - text_height) // 2
        
        if style.shadow:
            shadow_color = (*style.shadow_color, int(style.opacity * 128))
            draw.text(
                (x + style.shadow_offset[0], y + style.shadow_offset[1]),
                text, font=font, fill=shadow_color
            )
        
        draw.text((x, y), text, font=font, fill=style.rgba_color)
        
        return image
    
    def _add_positioned_watermark_pil(
        self,
        image: Image.Image,
        text: str,
        style: WatermarkStyle,
        position: str
    ) -> Image.Image:
        """添加定位水印"""
        width, height = image.size
        font = self._get_font(style)
        draw = ImageDraw.Draw(image)
        
        bbox = draw.textbbox((0, 0), text, font=font)
        text_width = bbox[2] - bbox[0]
        text_height = bbox[3] - bbox[1]
        
        margin = style.margin
        
        position_map = {
            "top_left": (margin, margin),
            "top_right": (width - text_width - margin, margin),
            "bottom_left": (margin, height - text_height - margin),
            "bottom_right": (width - text_width - margin, height - text_height - margin),
            "top_center": ((width - text_width) // 2, margin),
            "bottom_center": ((width - text_width) // 2, height - text_height - margin),
        }
        
        x, y = position_map.get(position, (margin, margin))
        
        # 绘制背景
        if style.background_color:
            padding = 5
            draw.rectangle(
                [x - padding, y - padding, x + text_width + padding, y + text_height + padding],
                fill=style.background_color
            )
        
        # 绘制边框
        if style.border:
            padding = 5
            draw.rectangle(
                [x - padding, y - padding, x + text_width + padding, y + text_height + padding],
                outline=(*style.border_color, int(style.opacity * 255)),
                width=style.border_width
            )
        
        # 绘制阴影
        if style.shadow:
            shadow_color = (*style.shadow_color, int(style.opacity * 128))
            draw.text(
                (x + style.shadow_offset[0], y + style.shadow_offset[1]),
                text, font=font, fill=shadow_color
            )
        
        # 绘制文本
        draw.text((x, y), text, font=font, fill=style.rgba_color)
        
        return image
    
    def _apply_watermark_to_frame(
        self,
        frame: np.ndarray,
        watermark_overlay: np.ndarray,
        opacity: float
    ) -> np.ndarray:
        """
        将水印应用到视频帧
        
        Args:
            frame: BGR格式的视频帧
            watermark_overlay: RGBA格式的水印层
            opacity: 透明度
            
        Returns:
            添加水印后的帧
        """
        # 转换帧为RGBA
        frame_rgba = cv2.cvtColor(frame, cv2.COLOR_BGR2RGBA)
        
        # 获取水印的alpha通道
        alpha = watermark_overlay[:, :, 3:4] / 255.0
        
        # 混合
        blended = frame_rgba.astype(float)
        watermark_rgb = watermark_overlay[:, :, :3]
        
        # 只在有水印的地方混合
        mask = alpha > 0
        blended[:, :, :3] = np.where(
            np.broadcast_to(mask, blended[:, :, :3].shape),
            frame_rgba[:, :, :3] * (1 - alpha) + watermark_rgb * alpha,
            frame_rgba[:, :, :3]
        )
        
        # 转回BGR
        result = cv2.cvtColor(blended.astype(np.uint8), cv2.COLOR_RGBA2BGR)
        
        return result
    
    def extract_frame(
        self,
        video_path: Union[str, Path],
        frame_number: int = 0
    ) -> Image.Image:
        """
        提取视频帧
        
        Args:
            video_path: 视频路径
            frame_number: 帧号
            
        Returns:
            PIL Image对象
        """
        cap = cv2.VideoCapture(str(video_path))
        
        if not cap.isOpened():
            raise ValueError(f"无法打开视频文件: {video_path}")
        
        cap.set(cv2.CAP_PROP_POS_FRAMES, frame_number)
        ret, frame = cap.read()
        cap.release()
        
        if not ret:
            raise ValueError(f"无法读取帧 {frame_number}")
        
        # 转换为RGB并创建PIL Image
        frame_rgb = cv2.cvtColor(frame, cv2.COLOR_BGR2RGB)
        return Image.fromarray(frame_rgb)
    
    def extract(
        self,
        video_path: Union[str, Path],
        frame_number: int = 0,
        use_ocr: bool = False
    ) -> Optional[WatermarkInfo]:
        """
        从视频中提取水印信息
        
        Args:
            video_path: 视频路径
            frame_number: 要分析的帧号
            use_ocr: 是否使用OCR
            
        Returns:
            提取的水印信息
        """
        # 提取帧
        frame = self.extract_frame(video_path, frame_number)
        
        # 保存临时文件
        with tempfile.NamedTemporaryFile(suffix='.png', delete=False) as tmp:
            frame.save(tmp.name)
            tmp_path = tmp.name
        
        try:
            # 使用图片水印处理器提取
            return self._image_watermark.extract(tmp_path, use_ocr)
        finally:
            os.unlink(tmp_path)
    
    def get_video_info(self, video_path: Union[str, Path]) -> dict:
        """
        获取视频信息
        
        Args:
            video_path: 视频路径
            
        Returns:
            视频信息字典
        """
        cap = cv2.VideoCapture(str(video_path))
        
        if not cap.isOpened():
            raise ValueError(f"无法打开视频文件: {video_path}")
        
        info = {
            "fps": cap.get(cv2.CAP_PROP_FPS),
            "width": int(cap.get(cv2.CAP_PROP_FRAME_WIDTH)),
            "height": int(cap.get(cv2.CAP_PROP_FRAME_HEIGHT)),
            "total_frames": int(cap.get(cv2.CAP_PROP_FRAME_COUNT)),
            "duration": cap.get(cv2.CAP_PROP_FRAME_COUNT) / cap.get(cv2.CAP_PROP_FPS),
            "codec": int(cap.get(cv2.CAP_PROP_FOURCC)),
        }
        
        cap.release()
        return info
    
    def batch_embed(
        self,
        input_dir: Union[str, Path],
        output_dir: Union[str, Path],
        info: WatermarkInfo,
        style: Optional[WatermarkStyle] = None,
        recursive: bool = False,
        progress_callback: Optional[Callable[[str, int, int], None]] = None
    ) -> List[str]:
        """
        批量添加水印
        
        Args:
            input_dir: 输入目录
            output_dir: 输出目录
            info: 水印信息
            style: 水印样式
            recursive: 是否递归处理
            progress_callback: 进度回调 (filename, current, total)
            
        Returns:
            处理的文件列表
        """
        input_dir = Path(input_dir)
        output_dir = Path(output_dir)
        
        if not input_dir.is_dir():
            raise ValueError(f"输入路径不是目录: {input_dir}")
        
        processed = []
        
        # 获取文件列表
        if recursive:
            files = list(input_dir.rglob("*"))
        else:
            files = list(input_dir.glob("*"))
        
        video_files = [f for f in files if f.suffix.lower() in self.SUPPORTED_FORMATS]
        total = len(video_files)
        
        for i, input_file in enumerate(video_files):
            relative_path = input_file.relative_to(input_dir)
            output_file = output_dir / relative_path
            
            try:
                if progress_callback:
                    progress_callback(str(input_file.name), i + 1, total)
                
                self.embed(input_file, output_file, info, style)
                processed.append(str(output_file))
            except Exception as e:
                print(f"处理失败 {input_file}: {e}")
        
        return processed



"""
核心模块
"""

from visualwm.core.info import WatermarkInfo
from visualwm.core.style import WatermarkStyle, RiskLevel
from visualwm.core.checksum import ChecksumGenerator
from visualwm.core.device import DeviceInfoCollector

__all__ = [
    "WatermarkInfo",
    "WatermarkStyle",
    "RiskLevel",
    "ChecksumGenerator",
    "DeviceInfoCollector",
]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
VisualWM 综合示例 - 完整功能演示
"""

from visualwm import (
    WatermarkInfo,
    WatermarkStyle,
    RiskLevel,
    ChecksumGenerator,
    ImageWatermark,
    VideoWatermark,
    TextWatermark,
)
from visualwm.core.device import DeviceInfoCollector
from visualwm.core.checksum import TamperDetector


def demo_device_info():
    """设备信息收集演示"""
    print("=" * 60)
    print("设备信息收集演示")
    print("=" * 60)
    
    collector = DeviceInfoCollector()
    
    print(f"IP地址: {collector.get_ip_address()}")
    print(f"MAC地址: {collector.get_mac_address()}")
    print(f"主机名: {collector.get_hostname()}")
    print(f"时间戳: {collector.get_timestamp()}")
    
    platform_info = collector.get_platform_info()
    print(f"操作系统: {platform_info['system']} {platform_info['release']}")
    print(f"机器类型: {platform_info['machine']}")
    
    print()


def demo_watermark_info():
    """水印信息演示"""
    print("=" * 60)
    print("水印信息演示")
    print("=" * 60)
    
    # 创建完整的水印信息
    info = WatermarkInfo(
        username="张三",
        employee_id="EMP20240001",
        device_id="DEV-LAPTOP-001",
        department="信息安全部",
        auto_fill=True,  # 自动填充IP、MAC、时间戳
        include_checksum=True,  # 包含校验码
        custom_data={
            "项目": "机密项目Alpha",
            "密级": "秘密"
        }
    )
    
    print("水印信息对象:")
    print(f"  用户名: {info.username}")
    print(f"  工号: {info.employee_id}")
    print(f"  设备ID: {info.device_id}")
    print(f"  部门: {info.department}")
    print(f"  IP地址: {info.ip_address}")
    print(f"  MAC地址: {info.mac_address}")
    print(f"  时间戳: {info.timestamp}")
    print(f"  校验码: {info.checksum[:16]}...")
    
    print("\n水印字符串格式:")
    print(f"  单行: {info.to_string()}")
    print(f"  紧凑: {info.to_string(compact=True)}")
    
    print("\n多行格式:")
    for line in info.to_multiline_string().split('\n'):
        print(f"  {line}")
    
    print()


def demo_watermark_styles():
    """水印样式演示"""
    print("=" * 60)
    print("水印样式演示")
    print("=" * 60)
    
    # 各种预设样式
    styles = {
        "默认样式": WatermarkStyle.default(),
        "低风险": WatermarkStyle.low_risk_preset(),
        "中风险": WatermarkStyle.medium_risk_preset(),
        "高风险": WatermarkStyle.high_risk_preset(),
        "极高风险": WatermarkStyle.critical_risk_preset(),
        "角落水印": WatermarkStyle.corner_preset(),
        "居中水印": WatermarkStyle.center_preset(),
    }
    
    for name, style in styles.items():
        print(f"\n{name}:")
        print(f"  字体大小: {style.font_size}")
        print(f"  颜色: {style.hex_color}")
        print(f"  透明度: {style.opacity}")
        print(f"  位置: {style.position}")
        print(f"  前缀: {style.prefix or '无'}")
    
    print()


def demo_checksum():
    """校验码演示"""
    print("=" * 60)
    print("校验码与防篡改演示")
    print("=" * 60)
    
    content = "用户:张三 | 工号:EMP001 | 时间:2024-01-01 12:00:00"
    
    # 生成校验码
    checksum = ChecksumGenerator.generate(content)
    short_checksum = ChecksumGenerator.generate_short(content, length=8)
    
    print(f"原始内容: {content}")
    print(f"完整校验码: {checksum}")
    print(f"短校验码: {short_checksum}")
    
    # 验证校验码
    is_valid = ChecksumGenerator.verify(content, checksum)
    print(f"校验结果: {'✓ 通过' if is_valid else '✗ 失败'}")
    
    # 模拟篡改
    tampered_content = content.replace("张三", "李四")
    is_valid_tampered = ChecksumGenerator.verify(tampered_content, checksum)
    print(f"\n篡改后内容: {tampered_content}")
    print(f"校验结果: {'✓ 通过' if is_valid_tampered else '✗ 失败（检测到篡改）'}")
    
    # 使用篡改检测器
    print("\n使用篡改检测器:")
    detector = TamperDetector()
    result = detector.check(content, tampered_content, short_checksum)
    
    print(f"  是否被篡改: {'是' if result['is_tampered'] else '否'}")
    print(f"  内容已修改: {'是' if result['content_modified'] else '否'}")
    print(f"  校验码有效: {'是' if result['checksum_valid'] else '否'}")
    if result['details']:
        print(f"  详情: {', '.join(result['details'])}")
    
    print()


def demo_complete_workflow():
    """完整工作流演示"""
    print("=" * 60)
    print("完整工作流演示")
    print("=" * 60)
    
    # 1. 创建水印信息
    print("\n步骤1: 创建水印信息")
    info = WatermarkInfo(
        username="安全审计员",
        employee_id="SEC001",
        department="安全合规部",
        auto_fill=True
    )
    print(f"  水印信息已创建: {info.username} ({info.employee_id})")
    
    # 2. 选择样式
    print("\n步骤2: 选择水印样式")
    style = WatermarkStyle.high_risk_preset()
    print(f"  使用高风险预设样式")
    print(f"  前缀: {style.prefix}")
    print(f"  颜色: {style.hex_color}")
    
    # 3. 初始化处理器
    print("\n步骤3: 初始化水印处理器")
    image_wm = ImageWatermark(default_style=style)
    video_wm = VideoWatermark(default_style=style)
    text_wm = TextWatermark()
    print("  ✓ 图片水印处理器")
    print("  ✓ 视频水印处理器")
    print("  ✓ 文本水印处理器")
    
    # 4. 添加水印
    print("\n步骤4: 添加水印")
    print("  图片: image_wm.embed('input.jpg', 'output.jpg', info)")
    print("  视频: video_wm.embed('input.mp4', 'output.mp4', info)")
    print("  文本: text_wm.embed('文本内容', info)")
    
    # 5. 提取和验证
    print("\n步骤5: 提取和验证水印")
    print("  图片: extracted = image_wm.extract('output.jpg', use_ocr=True)")
    print("  视频: extracted = video_wm.extract('output.mp4', use_ocr=True)")
    print("  文本: extracted = text_wm.extract(watermarked_text)")
    
    # 6. 演示文本水印完整流程
    print("\n步骤6: 文本水印完整流程演示")
    sample_text = "这是一份机密文档的内容示例。"
    watermarked = text_wm.embed(sample_text, info)
    extracted = text_wm.extract(watermarked)
    
    if extracted:
        print(f"  提取的用户: {extracted.username}")
        print(f"  提取的工号: {extracted.employee_id}")
        print("  ✓ 水印提取成功")
    
    print()


def demo_risk_levels():
    """风险等级演示"""
    print("=" * 60)
    print("风险等级水印演示")
    print("=" * 60)
    
    risk_descriptions = {
        RiskLevel.LOW: "普通文档 - 浅灰色半透明水印",
        RiskLevel.MEDIUM: "内部文档 - 带【内部】前缀",
        RiskLevel.HIGH: "机密文档 - 红色加粗，带【机密】前缀",
        RiskLevel.CRITICAL: "绝密文档 - 深红色加粗带阴影，【绝密】前缀",
    }
    
    for level, desc in risk_descriptions.items():
        style = WatermarkStyle.get_preset_by_risk_level(level)
        print(f"\n{level.value.upper()}: {desc}")
        print(f"  颜色: {style.hex_color}")
        print(f"  透明度: {style.opacity}")
        print(f"  字体大小: {style.font_size}")
        print(f"  加粗: {'是' if style.bold else '否'}")
        print(f"  前缀: {style.prefix or '无'}")
    
    print()


def main():
    """主函数"""
    print("\n" + "=" * 60)
    print("VisualWM - 泄密警示明水印SDK 综合演示")
    print("=" * 60 + "\n")
    
    demo_device_info()
    demo_watermark_info()
    demo_watermark_styles()
    demo_checksum()
    demo_risk_levels()
    demo_complete_workflow()
    
    print("=" * 60)
    print("演示完成!")
    print("=" * 60)
    print("\n使用方法:")
    print("  pip install -e .")
    print("  python -c 'from visualwm import ImageWatermark; print(\"OK\")'")
    print("\n命令行工具:")
    print("  visualwm embed input.jpg output.jpg -u 张三 -e EMP001")
    print("  visualwm extract output.jpg --ocr")
    print("  visualwm batch ./input_dir ./output_dir -u 张三 -t image")
    print()


if __name__ == "__main__":
    main()



"""
水印样式配置 - 定义水印的视觉属性
"""

from dataclasses import dataclass, field
from enum import Enum
from typing import Tuple, Optional, Dict, Any


class RiskLevel(Enum):
    """风险等级"""
    LOW = "low"           # 低风险 - 普通文档
    MEDIUM = "medium"     # 中风险 - 内部文档
    HIGH = "high"         # 高风险 - 机密文档
    CRITICAL = "critical" # 极高风险 - 绝密文档


class WatermarkPosition(Enum):
    """水印位置"""
    TILE = "tile"               # 平铺
    CENTER = "center"           # 居中
    TOP_LEFT = "top_left"       # 左上角
    TOP_RIGHT = "top_right"     # 右上角
    BOTTOM_LEFT = "bottom_left" # 左下角
    BOTTOM_RIGHT = "bottom_right" # 右下角
    TOP_CENTER = "top_center"   # 顶部居中
    BOTTOM_CENTER = "bottom_center" # 底部居中


@dataclass
class WatermarkStyle:
    """
    水印样式类
    
    用于配置水印的视觉属性，包括字体、颜色、透明度、位置等。
    
    Attributes:
        font_name: 字体名称
        font_size: 字体大小
        color: RGB颜色元组
        opacity: 透明度(0-1)
        rotation: 旋转角度
        position: 位置模式
        bold: 是否加粗
        italic: 是否斜体
        prefix: 前缀文字
        suffix: 后缀文字
        line_spacing: 行间距（平铺模式）
        column_spacing: 列间距（平铺模式）
        margin: 边距
        risk_level: 风险等级
        border: 是否添加边框
        background_color: 背景颜色
    """
    
    font_name: str = "SimHei"
    font_size: int = 24
    color: Tuple[int, int, int] = (128, 128, 128)
    opacity: float = 0.5
    rotation: int = -30
    position: str = "tile"
    bold: bool = False
    italic: bool = False
    prefix: str = ""
    suffix: str = ""
    line_spacing: int = 100
    column_spacing: int = 200
    margin: int = 20
    risk_level: RiskLevel = RiskLevel.LOW
    border: bool = False
    border_color: Tuple[int, int, int] = (200, 200, 200)
    border_width: int = 1
    background_color: Optional[Tuple[int, int, int, int]] = None
    shadow: bool = False
    shadow_color: Tuple[int, int, int] = (50, 50, 50)
    shadow_offset: Tuple[int, int] = (2, 2)
    
    def __post_init__(self):
        """初始化后验证"""
        self._validate()
    
    def _validate(self):
        """验证参数"""
        if not 0 <= self.opacity <= 1:
            raise ValueError("透明度必须在0-1之间")
        
        if self.font_size <= 0:
            raise ValueError("字体大小必须大于0")
        
        if not all(0 <= c <= 255 for c in self.color):
            raise ValueError("颜色值必须在0-255之间")
    
    @property
    def rgba_color(self) -> Tuple[int, int, int, int]:
        """获取RGBA颜色（带透明度）"""
        alpha = int(self.opacity * 255)
        return (*self.color, alpha)
    
    @property
    def hex_color(self) -> str:
        """获取十六进制颜色"""
        return "#{:02x}{:02x}{:02x}".format(*self.color)
    
    def get_font_style(self) -> str:
        """获取字体样式描述"""
        styles = []
        if self.bold:
            styles.append("bold")
        if self.italic:
            styles.append("italic")
        return " ".join(styles) if styles else "normal"
    
    def to_dict(self) -> Dict[str, Any]:
        """转换为字典"""
        return {
            "font_name": self.font_name,
            "font_size": self.font_size,
            "color": self.color,
            "opacity": self.opacity,
            "rotation": self.rotation,
            "position": self.position,
            "bold": self.bold,
            "italic": self.italic,
            "prefix": self.prefix,
            "suffix": self.suffix,
            "line_spacing": self.line_spacing,
            "column_spacing": self.column_spacing,
            "margin": self.margin,
            "risk_level": self.risk_level.value,
            "border": self.border,
            "border_color": self.border_color,
            "shadow": self.shadow,
        }
    
    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "WatermarkStyle":
        """从字典创建实例"""
        if "risk_level" in data and isinstance(data["risk_level"], str):
            data["risk_level"] = RiskLevel(data["risk_level"])
        return cls(**data)
    
    @classmethod
    def default(cls) -> "WatermarkStyle":
        """默认样式"""
        return cls()
    
    @classmethod
    def low_risk_preset(cls) -> "WatermarkStyle":
        """低风险预设样式"""
        return cls(
            font_size=20,
            color=(180, 180, 180),
            opacity=0.3,
            rotation=-30,
            position="tile",
            line_spacing=150,
            column_spacing=250,
            risk_level=RiskLevel.LOW,
        )
    
    @classmethod
    def medium_risk_preset(cls) -> "WatermarkStyle":
        """中风险预设样式"""
        return cls(
            font_size=24,
            color=(128, 128, 128),
            opacity=0.4,
            rotation=-30,
            position="tile",
            line_spacing=120,
            column_spacing=200,
            prefix="【内部】",
            risk_level=RiskLevel.MEDIUM,
        )
    
    @classmethod
    def high_risk_preset(cls) -> "WatermarkStyle":
        """高风险预设样式（涉密文档）"""
        return cls(
            font_size=28,
            color=(255, 0, 0),
            opacity=0.6,
            rotation=-30,
            position="tile",
            bold=True,
            line_spacing=100,
            column_spacing=180,
            prefix="【机密】",
            border=True,
            border_color=(255, 0, 0),
            risk_level=RiskLevel.HIGH,
        )
    
    @classmethod
    def critical_risk_preset(cls) -> "WatermarkStyle":
        """极高风险预设样式（绝密文档）"""
        return cls(
            font_size=32,
            color=(200, 0, 0),
            opacity=0.7,
            rotation=-30,
            position="tile",
            bold=True,
            line_spacing=80,
            column_spacing=150,
            prefix="【绝密】",
            border=True,
            border_color=(200, 0, 0),
            border_width=2,
            shadow=True,
            shadow_color=(100, 0, 0),
            risk_level=RiskLevel.CRITICAL,
        )
    
    @classmethod
    def get_preset_by_risk_level(cls, level: RiskLevel) -> "WatermarkStyle":
        """根据风险等级获取预设样式"""
        presets = {
            RiskLevel.LOW: cls.low_risk_preset,
            RiskLevel.MEDIUM: cls.medium_risk_preset,
            RiskLevel.HIGH: cls.high_risk_preset,
            RiskLevel.CRITICAL: cls.critical_risk_preset,
        }
        return presets[level]()
    
    @classmethod
    def corner_preset(cls, position: str = "bottom_right") -> "WatermarkStyle":
        """角落水印预设"""
        return cls(
            font_size=16,
            color=(100, 100, 100),
            opacity=0.6,
            rotation=0,
            position=position,
            margin=15,
        )
    
    @classmethod
    def center_preset(cls) -> "WatermarkStyle":
        """居中水印预设"""
        return cls(
            font_size=48,
            color=(150, 150, 150),
            opacity=0.3,
            rotation=-30,
            position="center",
        )
    
    def copy(self, **kwargs) -> "WatermarkStyle":
        """
        复制并修改样式
        
        Args:
            **kwargs: 要修改的属性
            
        Returns:
            新的WatermarkStyle实例
        """
        data = self.to_dict()
        data.update(kwargs)
        return self.from_dict(data)



"""
文档水印模块 - 支持Word、PDF、Excel、OFD等格式
"""

from visualwm.document.word import WordWatermark
from visualwm.document.pdf import PDFWatermark
from visualwm.document.excel import ExcelWatermark
from visualwm.document.ofd import OFDWatermark

__all__ = [
    "WordWatermark",
    "PDFWatermark",
    "ExcelWatermark",
    "OFDWatermark",
]



"""
文本水印处理器 - 支持文本内容添加明水印和提取
"""

import re
import json
from pathlib import Path
from typing import Optional, Union, Dict, Any, List
from dataclasses import dataclass
from enum import Enum

from visualwm.core.info import WatermarkInfo
from visualwm.core.style import WatermarkStyle, RiskLevel


class TextWatermarkFormat(Enum):
    """文本水印格式"""
    HEADER = "header"           # 头部水印
    FOOTER = "footer"           # 尾部水印
    HEADER_FOOTER = "header_footer"  # 头尾水印
    INLINE = "inline"           # 行内水印
    MARGIN = "margin"           # 边距水印
    BRACKET = "bracket"         # 括号包围


@dataclass
class TextWatermarkStyle:
    """
    文本水印样式
    
    Attributes:
        format: 水印格式
        separator: 分隔符
        line_prefix: 行前缀（边距水印）
        bracket_left: 左括号
        bracket_right: 右括号
        repeat_interval: 重复间隔（行内水印）
        include_border: 是否包含边框
        border_char: 边框字符
        risk_level: 风险等级
    """
    format: TextWatermarkFormat = TextWatermarkFormat.HEADER_FOOTER
    separator: str = "\n"
    line_prefix: str = "| "
    bracket_left: str = "【"
    bracket_right: str = "】"
    repeat_interval: int = 10
    include_border: bool = True
    border_char: str = "="
    border_length: int = 60
    risk_level: RiskLevel = RiskLevel.LOW
    prefix: str = ""
    
    @classmethod
    def header_preset(cls) -> "TextWatermarkStyle":
        """头部水印预设"""
        return cls(format=TextWatermarkFormat.HEADER)
    
    @classmethod
    def footer_preset(cls) -> "TextWatermarkStyle":
        """尾部水印预设"""
        return cls(format=TextWatermarkFormat.FOOTER)
    
    @classmethod
    def high_risk_preset(cls) -> "TextWatermarkStyle":
        """高风险预设"""
        return cls(
            format=TextWatermarkFormat.HEADER_FOOTER,
            include_border=True,
            border_char="*",
            risk_level=RiskLevel.HIGH,
            prefix="【机密】"
        )


class TextWatermark:
    """
    文本水印处理器
    
    支持在纯文本内容中添加明水印，并从文本中提取水印信息。
    
    Features:
        - 多种水印格式：头部、尾部、行内、边距
        - 风险等级标识
        - 水印信息提取
        - 防篡改校验
    """
    
    # 水印标记
    WATERMARK_START_MARKER = "<!-- VISUALWM_START -->"
    WATERMARK_END_MARKER = "<!-- VISUALWM_END -->"
    
    def __init__(self, default_style: Optional[TextWatermarkStyle] = None):
        """
        初始化文本水印处理器
        
        Args:
            default_style: 默认水印样式
        """
        self.default_style = default_style or TextWatermarkStyle()
    
    def embed(
        self,
        content: str,
        info: WatermarkInfo,
        style: Optional[TextWatermarkStyle] = None,
    ) -> str:
        """
        在文本内容中嵌入水印
        
        Args:
            content: 原始文本内容
            info: 水印信息
            style: 水印样式
            
        Returns:
            带水印的文本内容
        """
        style = style or self.default_style
        
        # 生成水印文本
        watermark_text = self._generate_watermark_text(info, style)
        
        # 根据格式添加水印
        format_type = style.format
        
        if format_type == TextWatermarkFormat.HEADER:
            return self._add_header_watermark(content, watermark_text, style)
        
        elif format_type == TextWatermarkFormat.FOOTER:
            return self._add_footer_watermark(content, watermark_text, style)
        
        elif format_type == TextWatermarkFormat.HEADER_FOOTER:
            return self._add_header_footer_watermark(content, watermark_text, style)
        
        elif format_type == TextWatermarkFormat.INLINE:
            return self._add_inline_watermark(content, watermark_text, style)
        
        elif format_type == TextWatermarkFormat.MARGIN:
            return self._add_margin_watermark(content, watermark_text, style)
        
        elif format_type == TextWatermarkFormat.BRACKET:
            return self._add_bracket_watermark(content, watermark_text, style)
        
        else:
            raise ValueError(f"不支持的水印格式: {format_type}")
    
    def _generate_watermark_text(
        self,
        info: WatermarkInfo,
        style: TextWatermarkStyle
    ) -> str:
        """生成水印文本"""
        text = info.to_string(separator=" | ")
        
        if style.prefix:
            text = f"{style.prefix} {text}"
        
        return text
    
    def _create_border(self, style: TextWatermarkStyle) -> str:
        """创建边框"""
        return style.border_char * style.border_length
    
    def _add_header_watermark(
        self,
        content: str,
        watermark_text: str,
        style: TextWatermarkStyle
    ) -> str:
        """添加头部水印"""
        parts = []
        
        if style.include_border:
            parts.append(self._create_border(style))
        
        parts.append(self.WATERMARK_START_MARKER)
        parts.append(watermark_text)
        parts.append(self.WATERMARK_END_MARKER)
        
        if style.include_border:
            parts.append(self._create_border(style))
        
        parts.append("")
        parts.append(content)
        
        return "\n".join(parts)
    
    def _add_footer_watermark(
        self,
        content: str,
        watermark_text: str,
        style: TextWatermarkStyle
    ) -> str:
        """添加尾部水印"""
        parts = [content, ""]
        
        if style.include_border:
            parts.append(self._create_border(style))
        
        parts.append(self.WATERMARK_START_MARKER)
        parts.append(watermark_text)
        parts.append(self.WATERMARK_END_MARKER)
        
        if style.include_border:
            parts.append(self._create_border(style))
        
        return "\n".join(parts)
    
    def _add_header_footer_watermark(
        self,
        content: str,
        watermark_text: str,
        style: TextWatermarkStyle
    ) -> str:
        """添加头尾水印"""
        parts = []
        
        # 头部
        if style.include_border:
            parts.append(self._create_border(style))
        
        parts.append(self.WATERMARK_START_MARKER)
        parts.append(watermark_text)
        parts.append(self.WATERMARK_END_MARKER)
        
        if style.include_border:
            parts.append(self._create_border(style))
        
        parts.append("")
        parts.append(content)
        parts.append("")
        
        # 尾部
        if style.include_border:
            parts.append(self._create_border(style))
        
        parts.append(self.WATERMARK_START_MARKER)
        parts.append(watermark_text)
        parts.append(self.WATERMARK_END_MARKER)
        
        if style.include_border:
            parts.append(self._create_border(style))
        
        return "\n".join(parts)
    
    def _add_inline_watermark(
        self,
        content: str,
        watermark_text: str,
        style: TextWatermarkStyle
    ) -> str:
        """添加行内水印"""
        lines = content.split("\n")
        result = []
        
        inline_marker = f" {style.bracket_left}{watermark_text}{style.bracket_right}"
        
        for i, line in enumerate(lines):
            if i > 0 and i % style.repeat_interval == 0:
                result.append(line + inline_marker)
            else:
                result.append(line)
        
        return "\n".join(result)
    
    def _add_margin_watermark(
        self,
        content: str,
        watermark_text: str,
        style: TextWatermarkStyle
    ) -> str:
        """添加边距水印"""
        lines = content.split("\n")
        
        # 计算最长行
        max_length = max(len(line) for line in lines) if lines else 0
        
        # 添加头部水印
        header = f"{style.line_prefix}{watermark_text}"
        border = style.line_prefix + style.border_char * (max_length + 20)
        
        result = [border, header, border]
        
        for line in lines:
            result.append(f"{style.line_prefix}{line}")
        
        result.append(border)
        result.append(header)
        result.append(border)
        
        return "\n".join(result)
    
    def _add_bracket_watermark(
        self,
        content: str,
        watermark_text: str,
        style: TextWatermarkStyle
    ) -> str:
        """添加括号包围水印"""
        marker = f"{style.bracket_left}{watermark_text}{style.bracket_right}"
        
        return f"{marker}\n\n{content}\n\n{marker}"
    
    def embed_to_file(
        self,
        input_path: Union[str, Path],
        output_path: Union[str, Path],
        info: WatermarkInfo,
        style: Optional[TextWatermarkStyle] = None,
        encoding: str = "utf-8"
    ) -> str:
        """
        为文本文件添加水印
        
        Args:
            input_path: 输入文件路径
            output_path: 输出文件路径
            info: 水印信息
            style: 水印样式
            encoding: 文件编码
            
        Returns:
            输出文件路径
        """
        input_path = Path(input_path)
        output_path = Path(output_path)
        
        # 读取文件
        with open(input_path, 'r', encoding=encoding) as f:
            content = f.read()
        
        # 添加水印
        watermarked = self.embed(content, info, style)
        
        # 写入文件
        output_path.parent.mkdir(parents=True, exist_ok=True)
        with open(output_path, 'w', encoding=encoding) as f:
            f.write(watermarked)
        
        return str(output_path)
    
    def extract(self, content: str) -> Optional[WatermarkInfo]:
        """
        从文本中提取水印信息
        
        Args:
            content: 带水印的文本内容
            
        Returns:
            提取的水印信息，如果未找到返回None
        """
        # 尝试使用标记提取
        pattern = f"{re.escape(self.WATERMARK_START_MARKER)}\\s*(.+?)\\s*{re.escape(self.WATERMARK_END_MARKER)}"
        match = re.search(pattern, content, re.DOTALL)
        
        if match:
            watermark_text = match.group(1).strip()
            return WatermarkInfo.from_string(watermark_text)
        
        # 尝试使用括号提取
        bracket_pattern = r"【(.+?)】"
        matches = re.findall(bracket_pattern, content)
        
        for match in matches:
            if "用户:" in match or "工号:" in match or "IP:" in match:
                return WatermarkInfo.from_string(match)
        
        return None
    
    def extract_from_file(
        self,
        file_path: Union[str, Path],
        encoding: str = "utf-8"
    ) -> Optional[WatermarkInfo]:
        """
        从文本文件中提取水印信息
        
        Args:
            file_path: 文件路径
            encoding: 文件编码
            
        Returns:
            提取的水印信息
        """
        with open(file_path, 'r', encoding=encoding) as f:
            content = f.read()
        
        return self.extract(content)
    
    def remove_watermark(self, content: str) -> str:
        """
        移除文本中的水印
        
        Args:
            content: 带水印的文本
            
        Returns:
            移除水印后的文本
            
        Note:
            此功能主要用于测试，实际应用中不建议使用
        """
        # 移除标记水印
        pattern = f"{re.escape(self.WATERMARK_START_MARKER)}.*?{re.escape(self.WATERMARK_END_MARKER)}"
        content = re.sub(pattern, "", content, flags=re.DOTALL)
        
        # 移除边框
        content = re.sub(r"^[=*\-]{20,}\n", "", content, flags=re.MULTILINE)
        
        # 清理空行
        content = re.sub(r"\n{3,}", "\n\n", content)
        
        return content.strip()
    
    def verify_watermark(
        self,
        content: str,
        expected_info: WatermarkInfo
    ) -> Dict[str, Any]:
        """
        验证水印完整性
        
        Args:
            content: 带水印的文本
            expected_info: 预期的水印信息
            
        Returns:
            验证结果字典
        """
        result = {
            "is_valid": False,
            "watermark_found": False,
            "checksum_valid": False,
            "info_match": False,
            "details": []
        }
        
        # 提取水印
        extracted = self.extract(content)
        
        if extracted is None:
            result["details"].append("未找到水印")
            return result
        
        result["watermark_found"] = True
        
        # 验证校验码
        if extracted.checksum and expected_info.checksum:
            if extracted.checksum[:8] == expected_info.checksum[:8]:
                result["checksum_valid"] = True
            else:
                result["details"].append("校验码不匹配")
        
        # 验证关键信息
        if (extracted.username == expected_info.username and
            extracted.employee_id == expected_info.employee_id):
            result["info_match"] = True
        else:
            result["details"].append("用户信息不匹配")
        
        result["is_valid"] = result["checksum_valid"] and result["info_match"]
        
        return result
    
    def batch_embed(
        self,
        input_dir: Union[str, Path],
        output_dir: Union[str, Path],
        info: WatermarkInfo,
        style: Optional[TextWatermarkStyle] = None,
        file_extensions: List[str] = None,
        recursive: bool = False,
        encoding: str = "utf-8"
    ) -> List[str]:
        """
        批量添加水印
        
        Args:
            input_dir: 输入目录
            output_dir: 输出目录
            info: 水印信息
            style: 水印样式
            file_extensions: 文件扩展名列表
            recursive: 是否递归处理
            encoding: 文件编码
            
        Returns:
            处理的文件列表
        """
        input_dir = Path(input_dir)
        output_dir = Path(output_dir)
        
        if file_extensions is None:
            file_extensions = ['.txt', '.md', '.rst', '.log']
        
        processed = []
        
        # 获取文件列表
        if recursive:
            files = input_dir.rglob("*")
        else:
            files = input_dir.glob("*")
        
        for input_file in files:
            if input_file.suffix.lower() in file_extensions:
                relative_path = input_file.relative_to(input_dir)
                output_file = output_dir / relative_path
                
                try:
                    self.embed_to_file(input_file, output_file, info, style, encoding)
                    processed.append(str(output_file))
                except Exception as e:
                    print(f"处理失败 {input_file}: {e}")
        
        return processed


class HTMLWatermark(TextWatermark):
    """
    HTML水印处理器
    
    专门用于处理HTML内容的水印。
    """
    
    def embed(
        self,
        content: str,
        info: WatermarkInfo,
        style: Optional[TextWatermarkStyle] = None,
    ) -> str:
        """在HTML内容中嵌入水印"""
        style = style or self.default_style
        watermark_text = self._generate_watermark_text(info, style)
        
        # 创建HTML水印元素
        watermark_html = self._create_html_watermark(watermark_text, style)
        
        # 插入到body后面
        if "</body>" in content.lower():
            content = re.sub(
                r"(</body>)",
                f"{watermark_html}\\1",
                content,
                flags=re.IGNORECASE
            )
        else:
            content = f"{content}\n{watermark_html}"
        
        return content
    
    def _create_html_watermark(
        self,
        watermark_text: str,
        style: TextWatermarkStyle
    ) -> str:
        """创建HTML水印元素"""
        color = "red" if style.risk_level in [RiskLevel.HIGH, RiskLevel.CRITICAL] else "gray"
        opacity = "0.7" if style.risk_level in [RiskLevel.HIGH, RiskLevel.CRITICAL] else "0.5"
        
        return f'''
<!-- VISUALWM_START -->
<div class="visualwm-watermark" style="
    position: fixed;
    top: 0;
    left: 0;
    width: 100%;
    height: 100%;
    pointer-events: none;
    z-index: 999999;
    background: repeating-linear-gradient(
        -45deg,
        transparent,
        transparent 100px,
        rgba(0,0,0,0.02) 100px,
        rgba(0,0,0,0.02) 200px
    );
">
    <div style="
        position: absolute;
        top: 50%;
        left: 50%;
        transform: translate(-50%, -50%) rotate(-30deg);
        font-size: 24px;
        color: {color};
        opacity: {opacity};
        white-space: nowrap;
        font-family: Arial, sans-serif;
    ">
        {watermark_text}
    </div>
</div>
<!-- VISUALWM_END -->
'''


class MarkdownWatermark(TextWatermark):
    """
    Markdown水印处理器
    
    专门用于处理Markdown内容的水印。
    """
    
    def embed(
        self,
        content: str,
        info: WatermarkInfo,
        style: Optional[TextWatermarkStyle] = None,
    ) -> str:
        """在Markdown内容中嵌入水印"""
        style = style or self.default_style
        watermark_text = self._generate_watermark_text(info, style)
        
        # 创建Markdown水印
        warning = ""
        if style.risk_level in [RiskLevel.HIGH, RiskLevel.CRITICAL]:
            warning = "⚠️ **机密文档** ⚠️\n\n"
        
        header = f"""
---

{warning}> **文档水印信息**
> {watermark_text}

---

"""
        
        footer = f"""

---

> **文档水印信息**
> {watermark_text}

---
"""
        
        return f"{header}{content}{footer}"



"""
Word文档水印处理器 - 支持.docx格式添加明水印

注意：仅支持.docx格式（Office Open XML）
      不支持老的.doc格式（Office 97-2003二进制格式）
      如需处理.doc文件，请先转换为.docx格式
"""

import os
from pathlib import Path
from typing import Optional, Union, List
from docx import Document
from docx.shared import Pt, RGBColor, Inches, Emu
from docx.oxml.ns import qn, nsmap
from docx.oxml import OxmlElement
from docx.enum.text import WD_ALIGN_PARAGRAPH
from docx.enum.section import WD_ORIENT

from visualwm.core.info import WatermarkInfo
from visualwm.core.style import WatermarkStyle, RiskLevel


class WordWatermark:
    """
    Word文档水印处理器
    
    支持在Word文档(.docx)中添加明水印，包括：
    - 页眉/页脚水印
    - 对角线文字水印
    - 背景水印图层
    
    Features:
        - 支持.docx格式（Office 2007及以上）
        - 不支持.doc格式（需先转换为.docx）
        - 多种水印位置
        - 自定义样式
        - 批量处理
    
    Note:
        老的.doc格式（Office 97-2003）是微软专有的二进制格式，
        无法用纯Python处理。如需处理.doc文件，请使用以下方式转换：
        - Microsoft Word: 另存为 .docx
        - LibreOffice: 打开后另存为 .docx
        - 在线工具: 如 CloudConvert
    """
    
    SUPPORTED_EXTENSIONS = {'.docx'}
    
    def __init__(self, default_style: Optional[WatermarkStyle] = None):
        """
        初始化Word水印处理器
        
        Args:
            default_style: 默认水印样式
        """
        self.default_style = default_style or WatermarkStyle.default()
    
    def embed(
        self,
        input_path: Union[str, Path],
        output_path: Union[str, Path],
        info: WatermarkInfo,
        style: Optional[WatermarkStyle] = None,
        watermark_type: str = "header_footer"
    ) -> str:
        """
        在Word文档中嵌入水印
        
        Args:
            input_path: 输入文档路径
            output_path: 输出文档路径
            info: 水印信息
            style: 水印样式
            watermark_type: 水印类型 ("header_footer", "diagonal", "background")
            
        Returns:
            输出文件路径
        """
        input_path = Path(input_path)
        output_path = Path(output_path)
        style = style or self.default_style
        
        if not input_path.exists():
            raise FileNotFoundError(f"输入文件不存在: {input_path}")
        
        # 检查文件格式
        if input_path.suffix.lower() == '.doc':
            raise ValueError(
                f"不支持.doc格式（Office 97-2003二进制格式）\n"
                f"请先将文件转换为.docx格式：\n"
                f"  - Microsoft Word: 文件 > 另存为 > .docx\n"
                f"  - LibreOffice: 文件 > 另存为 > .docx\n"
                f"  - 在线工具: CloudConvert, Zamzar等"
            )
        
        if input_path.suffix.lower() not in self.SUPPORTED_EXTENSIONS:
            raise ValueError(f"不支持的文件格式: {input_path.suffix}，仅支持: {self.SUPPORTED_EXTENSIONS}")
        
        # 打开文档
        doc = Document(input_path)
        
        # 根据类型添加水印
        if watermark_type == "header_footer":
            self._add_header_footer_watermark(doc, info, style)
        elif watermark_type == "diagonal":
            self._add_diagonal_watermark(doc, info, style)
        elif watermark_type == "background":
            self._add_background_watermark(doc, info, style)
        else:
            raise ValueError(f"不支持的水印类型: {watermark_type}")
        
        # 保存文档
        output_path.parent.mkdir(parents=True, exist_ok=True)
        doc.save(output_path)
        
        return str(output_path)
    
    def embed_new(
        self,
        output_path: Union[str, Path],
        content: str,
        info: WatermarkInfo,
        style: Optional[WatermarkStyle] = None,
    ) -> str:
        """
        创建带水印的新Word文档
        
        Args:
            output_path: 输出文档路径
            content: 文档内容
            info: 水印信息
            style: 水印样式
            
        Returns:
            输出文件路径
        """
        output_path = Path(output_path)
        style = style or self.default_style
        
        # 创建新文档
        doc = Document()
        
        # 添加内容
        for paragraph in content.split('\n'):
            doc.add_paragraph(paragraph)
        
        # 添加水印
        self._add_header_footer_watermark(doc, info, style)
        
        # 保存
        output_path.parent.mkdir(parents=True, exist_ok=True)
        doc.save(output_path)
        
        return str(output_path)
    
    def _add_header_footer_watermark(
        self,
        doc: Document,
        info: WatermarkInfo,
        style: WatermarkStyle
    ):
        """添加页眉页脚水印"""
        watermark_text = self._format_watermark_text(info, style)
        
        # 获取或创建页眉页脚
        for section in doc.sections:
            # 页眉
            header = section.header
            header_para = header.paragraphs[0] if header.paragraphs else header.add_paragraph()
            header_para.clear()
            
            # 添加水印文本
            run = header_para.add_run(watermark_text)
            run.font.size = Pt(style.font_size * 0.5)  # 页眉字体稍小
            run.font.color.rgb = RGBColor(*style.color)
            run.font.bold = style.bold
            run.font.italic = style.italic
            header_para.alignment = WD_ALIGN_PARAGRAPH.CENTER
            
            # 页脚
            footer = section.footer
            footer_para = footer.paragraphs[0] if footer.paragraphs else footer.add_paragraph()
            footer_para.clear()
            
            run = footer_para.add_run(watermark_text)
            run.font.size = Pt(style.font_size * 0.5)
            run.font.color.rgb = RGBColor(*style.color)
            run.font.bold = style.bold
            run.font.italic = style.italic
            footer_para.alignment = WD_ALIGN_PARAGRAPH.CENTER
    
    def _add_diagonal_watermark(
        self,
        doc: Document,
        info: WatermarkInfo,
        style: WatermarkStyle
    ):
        """添加对角线文字水印（在页眉中实现）"""
        watermark_text = self._format_watermark_text(info, style)
        
        # 为每个section添加水印
        for section in doc.sections:
            self._add_text_watermark_to_header(section, watermark_text, style)
    
    def _add_text_watermark_to_header(self, section, text: str, style: WatermarkStyle):
        """在页眉中添加文字水印"""
        from lxml import etree
        
        # 获取页眉
        header = section.header
        header.is_linked_to_previous = False
        
        # 清除现有内容
        for para in header.paragraphs:
            para.clear()
        
        # VML命名空间
        NSMAP = {
            'w': 'http://schemas.openxmlformats.org/wordprocessingml/2006/main',
            'v': 'urn:schemas-microsoft-com:vml',
            'o': 'urn:schemas-microsoft-com:office:office',
            'w10': 'urn:schemas-microsoft-com:office:word',
        }
        
        # 创建水印XML
        r_elem = etree.SubElement(header.paragraphs[0]._p, '{%s}r' % NSMAP['w'])
        
        pict = etree.SubElement(r_elem, '{%s}pict' % NSMAP['w'])
        
        # 创建VML shape
        shape = etree.SubElement(pict, '{%s}shape' % NSMAP['v'], nsmap={'v': NSMAP['v'], 'o': NSMAP['o']})
        shape.set('id', 'PowerPlusWaterMarkObject')
        shape.set('type', '#_x0000_t136')
        shape.set('style', 
            f'position:absolute;'
            f'margin-left:0;margin-top:0;'
            f'width:500pt;height:150pt;'
            f'rotation:{style.rotation};'
            f'z-index:-251657216;'
            f'mso-position-horizontal:center;'
            f'mso-position-horizontal-relative:margin;'
            f'mso-position-vertical:center;'
            f'mso-position-vertical-relative:margin'
        )
        shape.set('fillcolor', f'#{style.color[0]:02x}{style.color[1]:02x}{style.color[2]:02x}')
        shape.set('stroked', 'f')
        shape.set('{%s}allowincell' % NSMAP['o'], 'f')
        
        # 添加fill
        fill = etree.SubElement(shape, '{%s}fill' % NSMAP['v'])
        fill.set('opacity', f'{style.opacity}')
        
        # 添加textpath
        textpath = etree.SubElement(shape, '{%s}textpath' % NSMAP['v'])
        textpath.set('style', f'font-family:"{style.font_name}";font-size:{style.font_size}pt')
        textpath.set('string', text)
    
    def _add_background_watermark(
        self,
        doc: Document,
        info: WatermarkInfo,
        style: WatermarkStyle
    ):
        """添加背景水印（每页重复）"""
        # 使用页眉页脚实现背景效果
        self._add_header_footer_watermark(doc, info, style)
        
        # 在文档开头添加水印提示
        watermark_text = self._format_watermark_text(info, style)
        
        # 在每个段落前插入水印
        # 注意：这会影响文档结构，仅用于演示
        first_para = doc.paragraphs[0] if doc.paragraphs else doc.add_paragraph()
        
        # 添加水印段落
        watermark_para = doc.add_paragraph()
        watermark_para.alignment = WD_ALIGN_PARAGRAPH.CENTER
        
        run = watermark_para.add_run(f"【{style.prefix or '文档水印'}】{watermark_text}")
        run.font.size = Pt(style.font_size)
        run.font.color.rgb = RGBColor(*style.color)
        run.font.bold = style.bold
        
        # 移动到开头
        first_para._element.addprevious(watermark_para._element)
    
    def _format_watermark_text(self, info: WatermarkInfo, style: WatermarkStyle) -> str:
        """格式化水印文本"""
        text = info.to_string(separator=" | ", compact=True)
        
        if style.prefix:
            text = f"{style.prefix} {text}"
        if style.suffix:
            text = f"{text} {style.suffix}"
        
        return text
    
    def extract(
        self,
        doc_path: Union[str, Path]
    ) -> Optional[WatermarkInfo]:
        """
        从Word文档中提取水印信息
        
        Args:
            doc_path: 文档路径
            
        Returns:
            提取的水印信息
        """
        doc = Document(doc_path)
        
        # 从页眉提取
        for section in doc.sections:
            header = section.header
            for para in header.paragraphs:
                text = para.text.strip()
                if text and ("用户:" in text or "工号:" in text):
                    return WatermarkInfo.from_string(text)
            
            # 从页脚提取
            footer = section.footer
            for para in footer.paragraphs:
                text = para.text.strip()
                if text and ("用户:" in text or "工号:" in text):
                    return WatermarkInfo.from_string(text)
        
        # 从正文提取
        for para in doc.paragraphs:
            text = para.text.strip()
            if "用户:" in text or "工号:" in text:
                return WatermarkInfo.from_string(text)
        
        return None
    
    def batch_embed(
        self,
        input_dir: Union[str, Path],
        output_dir: Union[str, Path],
        info: WatermarkInfo,
        style: Optional[WatermarkStyle] = None,
        recursive: bool = False
    ) -> List[str]:
        """
        批量添加水印
        
        Args:
            input_dir: 输入目录
            output_dir: 输出目录
            info: 水印信息
            style: 水印样式
            recursive: 是否递归处理
            
        Returns:
            处理的文件列表
        """
        input_dir = Path(input_dir)
        output_dir = Path(output_dir)
        
        processed = []
        
        pattern = "**/*.docx" if recursive else "*.docx"
        
        for input_file in input_dir.glob(pattern):
            if input_file.name.startswith('~$'):  # 跳过临时文件
                continue
            
            relative_path = input_file.relative_to(input_dir)
            output_file = output_dir / relative_path
            
            try:
                self.embed(input_file, output_file, info, style)
                processed.append(str(output_file))
            except Exception as e:
                print(f"处理失败 {input_file}: {e}")
        
        return processed



"""
OFD文档水印处理器 - 支持.ofd格式添加明水印

OFD (Open Fixed-layout Document) 是中国电子公文交换的版式文档格式标准。
OFD文件本质上是一个ZIP压缩包，包含XML描述文件和资源文件。
"""

import os
import io
import zipfile
import tempfile
import shutil
import xml.etree.ElementTree as ET
from pathlib import Path
from typing import Optional, Union, List, Dict, Any
from datetime import datetime

from visualwm.core.info import WatermarkInfo
from visualwm.core.style import WatermarkStyle, RiskLevel


class OFDWatermark:
    """
    OFD文档水印处理器
    
    支持在OFD文档中添加明水印。
    
    OFD文件结构：
    - OFD.xml: 入口文件
    - Doc_0/Document.xml: 文档描述
    - Doc_0/Pages/Page_0/Content.xml: 页面内容
    - Doc_0/Res/: 资源文件
    
    Features:
        - 支持.ofd格式
        - 文本水印层
        - 注释水印
        - 批量处理
        
    Note:
        OFD格式相对复杂，此实现提供基本的水印功能。
        对于更复杂的需求，建议使用专业的OFD处理库。
    """
    
    SUPPORTED_EXTENSIONS = {'.ofd'}
    
    # OFD命名空间
    OFD_NAMESPACE = "http://www.ofdspec.org/2016"
    
    def __init__(self, default_style: Optional[WatermarkStyle] = None):
        """
        初始化OFD水印处理器
        
        Args:
            default_style: 默认水印样式
        """
        self.default_style = default_style or WatermarkStyle.default()
        
        # 注册命名空间
        ET.register_namespace('ofd', self.OFD_NAMESPACE)
    
    def embed(
        self,
        input_path: Union[str, Path],
        output_path: Union[str, Path],
        info: WatermarkInfo,
        style: Optional[WatermarkStyle] = None,
        watermark_type: str = "annotation"
    ) -> str:
        """
        在OFD文档中嵌入水印
        
        Args:
            input_path: 输入OFD路径
            output_path: 输出OFD路径
            info: 水印信息
            style: 水印样式
            watermark_type: 水印类型 ("annotation", "layer", "metadata")
            
        Returns:
            输出文件路径
        """
        input_path = Path(input_path)
        output_path = Path(output_path)
        style = style or self.default_style
        
        if not input_path.exists():
            raise FileNotFoundError(f"输入文件不存在: {input_path}")
        
        # 创建临时目录
        with tempfile.TemporaryDirectory() as temp_dir:
            temp_path = Path(temp_dir)
            
            # 解压OFD文件
            self._extract_ofd(input_path, temp_path)
            
            # 添加水印
            if watermark_type == "annotation":
                self._add_annotation_watermark(temp_path, info, style)
            elif watermark_type == "layer":
                self._add_layer_watermark(temp_path, info, style)
            elif watermark_type == "metadata":
                self._add_metadata_watermark(temp_path, info, style)
            else:
                raise ValueError(f"不支持的水印类型: {watermark_type}")
            
            # 重新打包OFD
            output_path.parent.mkdir(parents=True, exist_ok=True)
            self._pack_ofd(temp_path, output_path)
        
        return str(output_path)
    
    def _extract_ofd(self, ofd_path: Path, extract_path: Path):
        """解压OFD文件"""
        with zipfile.ZipFile(ofd_path, 'r') as zf:
            zf.extractall(extract_path)
    
    def _pack_ofd(self, source_path: Path, ofd_path: Path):
        """打包OFD文件"""
        with zipfile.ZipFile(ofd_path, 'w', zipfile.ZIP_DEFLATED) as zf:
            for file_path in source_path.rglob('*'):
                if file_path.is_file():
                    arcname = file_path.relative_to(source_path)
                    zf.write(file_path, arcname)
    
    def _add_annotation_watermark(
        self,
        ofd_path: Path,
        info: WatermarkInfo,
        style: WatermarkStyle
    ):
        """添加注释水印"""
        watermark_text = self._format_watermark_text(info, style)
        
        # 查找文档目录
        doc_dirs = list(ofd_path.glob("Doc_*"))
        
        for doc_dir in doc_dirs:
            # 创建或更新Annotations.xml
            annot_path = doc_dir / "Annots" / "Annotations.xml"
            annot_path.parent.mkdir(parents=True, exist_ok=True)
            
            # 创建注释XML
            root = ET.Element('{%s}Annotations' % self.OFD_NAMESPACE)
            
            # 添加水印注释
            annot = ET.SubElement(root, '{%s}Annotation' % self.OFD_NAMESPACE)
            annot.set('ID', 'watermark_annot')
            annot.set('Type', 'Watermark')
            annot.set('Creator', 'VisualWM')
            annot.set('LastModDate', datetime.now().strftime('%Y-%m-%d'))
            
            # 注释内容
            content = ET.SubElement(annot, '{%s}Content' % self.OFD_NAMESPACE)
            content.text = watermark_text
            
            # 保存
            tree = ET.ElementTree(root)
            tree.write(annot_path, encoding='utf-8', xml_declaration=True)
            
            # 更新Document.xml引用
            self._update_document_reference(doc_dir, "Annots/Annotations.xml")
    
    def _add_layer_watermark(
        self,
        ofd_path: Path,
        info: WatermarkInfo,
        style: WatermarkStyle
    ):
        """添加图层水印"""
        watermark_text = self._format_watermark_text(info, style)
        
        # 查找所有页面Content.xml
        for content_path in ofd_path.rglob("Content.xml"):
            try:
                tree = ET.parse(content_path)
                root = tree.getroot()
                
                # 添加水印文本对象
                # 注：实际OFD结构更复杂，这里是简化实现
                watermark_elem = ET.SubElement(root, '{%s}TextObject' % self.OFD_NAMESPACE)
                watermark_elem.set('ID', 'watermark_text')
                watermark_elem.set('Boundary', '10 10 200 20')
                
                text_code = ET.SubElement(watermark_elem, '{%s}TextCode' % self.OFD_NAMESPACE)
                text_code.text = watermark_text
                
                tree.write(content_path, encoding='utf-8', xml_declaration=True)
                
            except ET.ParseError:
                continue
    
    def _add_metadata_watermark(
        self,
        ofd_path: Path,
        info: WatermarkInfo,
        style: WatermarkStyle
    ):
        """添加元数据水印"""
        watermark_text = self._format_watermark_text(info, style)
        
        # 查找OFD.xml
        ofd_xml_path = ofd_path / "OFD.xml"
        
        if ofd_xml_path.exists():
            try:
                tree = ET.parse(ofd_xml_path)
                root = tree.getroot()
                
                # 添加自定义元数据
                custom_data = ET.SubElement(root, '{%s}CustomDatas' % self.OFD_NAMESPACE)
                
                watermark_data = ET.SubElement(custom_data, '{%s}CustomData' % self.OFD_NAMESPACE)
                watermark_data.set('Name', 'Watermark')
                watermark_data.text = watermark_text
                
                creator_data = ET.SubElement(custom_data, '{%s}CustomData' % self.OFD_NAMESPACE)
                creator_data.set('Name', 'WatermarkCreator')
                creator_data.text = 'VisualWM'
                
                timestamp_data = ET.SubElement(custom_data, '{%s}CustomData' % self.OFD_NAMESPACE)
                timestamp_data.set('Name', 'WatermarkTime')
                timestamp_data.text = datetime.now().isoformat()
                
                tree.write(ofd_xml_path, encoding='utf-8', xml_declaration=True)
                
            except ET.ParseError:
                pass
        
        # 同时添加注释水印作为可见标记
        self._add_annotation_watermark(ofd_path, info, style)
    
    def _update_document_reference(self, doc_dir: Path, ref_path: str):
        """更新文档引用"""
        doc_xml_path = doc_dir / "Document.xml"
        
        if doc_xml_path.exists():
            try:
                tree = ET.parse(doc_xml_path)
                root = tree.getroot()
                
                # 添加注释引用
                annots_ref = ET.SubElement(root, '{%s}Annotations' % self.OFD_NAMESPACE)
                annots_ref.text = ref_path
                
                tree.write(doc_xml_path, encoding='utf-8', xml_declaration=True)
                
            except ET.ParseError:
                pass
    
    def _format_watermark_text(self, info: WatermarkInfo, style: WatermarkStyle) -> str:
        """格式化水印文本"""
        text = info.to_string(separator=" | ", compact=True)
        
        if style.prefix:
            text = f"{style.prefix} {text}"
        if style.suffix:
            text = f"{text} {style.suffix}"
        
        return text
    
    def extract(
        self,
        ofd_path: Union[str, Path]
    ) -> Optional[WatermarkInfo]:
        """
        从OFD文档中提取水印信息
        
        Args:
            ofd_path: OFD文件路径
            
        Returns:
            提取的水印信息
        """
        ofd_path = Path(ofd_path)
        
        with tempfile.TemporaryDirectory() as temp_dir:
            temp_path = Path(temp_dir)
            self._extract_ofd(ofd_path, temp_path)
            
            # 从注释中提取
            for annot_path in temp_path.rglob("Annotations.xml"):
                try:
                    tree = ET.parse(annot_path)
                    root = tree.getroot()
                    
                    for content in root.iter('{%s}Content' % self.OFD_NAMESPACE):
                        text = content.text
                        if text and ("用户:" in text or "工号:" in text):
                            return WatermarkInfo.from_string(text)
                except ET.ParseError:
                    continue
            
            # 从元数据中提取
            ofd_xml_path = temp_path / "OFD.xml"
            if ofd_xml_path.exists():
                try:
                    tree = ET.parse(ofd_xml_path)
                    root = tree.getroot()
                    
                    for custom_data in root.iter('{%s}CustomData' % self.OFD_NAMESPACE):
                        if custom_data.get('Name') == 'Watermark':
                            text = custom_data.text
                            if text:
                                return WatermarkInfo.from_string(text)
                except ET.ParseError:
                    pass
        
        return None
    
    def get_ofd_info(self, ofd_path: Union[str, Path]) -> Dict[str, Any]:
        """
        获取OFD文档信息
        
        Args:
            ofd_path: OFD文件路径
            
        Returns:
            文档信息字典
        """
        ofd_path = Path(ofd_path)
        
        info = {
            "file_size": ofd_path.stat().st_size,
            "documents": [],
            "pages": 0,
        }
        
        with tempfile.TemporaryDirectory() as temp_dir:
            temp_path = Path(temp_dir)
            self._extract_ofd(ofd_path, temp_path)
            
            # 解析OFD.xml
            ofd_xml_path = temp_path / "OFD.xml"
            if ofd_xml_path.exists():
                try:
                    tree = ET.parse(ofd_xml_path)
                    root = tree.getroot()
                    
                    # 获取文档引用
                    for doc_ref in root.iter('{%s}DocRoot' % self.OFD_NAMESPACE):
                        info["documents"].append(doc_ref.text)
                except ET.ParseError:
                    pass
            
            # 统计页面数
            info["pages"] = len(list(temp_path.rglob("Page_*/Content.xml")))
        
        return info
    
    def batch_embed(
        self,
        input_dir: Union[str, Path],
        output_dir: Union[str, Path],
        info: WatermarkInfo,
        style: Optional[WatermarkStyle] = None,
        recursive: bool = False
    ) -> List[str]:
        """
        批量添加水印
        
        Args:
            input_dir: 输入目录
            output_dir: 输出目录
            info: 水印信息
            style: 水印样式
            recursive: 是否递归处理
            
        Returns:
            处理的文件列表
        """
        input_dir = Path(input_dir)
        output_dir = Path(output_dir)
        
        processed = []
        
        pattern = "**/*.ofd" if recursive else "*.ofd"
        
        for input_file in input_dir.glob(pattern):
            relative_path = input_file.relative_to(input_dir)
            output_file = output_dir / relative_path
            
            try:
                self.embed(input_file, output_file, info, style)
                processed.append(str(output_file))
            except Exception as e:
                print(f"处理失败 {input_file}: {e}")
        
        return processed
    
    def is_valid_ofd(self, ofd_path: Union[str, Path]) -> bool:
        """
        检查是否为有效的OFD文件
        
        Args:
            ofd_path: OFD文件路径
            
        Returns:
            是否有效
        """
        try:
            with zipfile.ZipFile(ofd_path, 'r') as zf:
                # 检查是否包含OFD.xml
                namelist = zf.namelist()
                return 'OFD.xml' in namelist
        except (zipfile.BadZipFile, FileNotFoundError):
            return False



"""
水印信息模型 - 存储溯源信息
"""

from dataclasses import dataclass, field
from typing import Dict, Optional, Any
from datetime import datetime

from visualwm.core.device import DeviceInfoCollector
from visualwm.core.checksum import ChecksumGenerator


@dataclass
class WatermarkInfo:
    """
    水印信息类
    
    用于存储和管理水印中嵌入的溯源信息，包括用户信息、设备信息、时间戳等。
    
    Attributes:
        username: 用户姓名
        employee_id: 工号
        device_id: 设备编号
        ip_address: IP地址
        mac_address: MAC地址
        timestamp: 时间戳
        custom_data: 自定义扩展数据
        auto_fill: 是否自动填充动态信息
        include_checksum: 是否包含校验码
    """
    
    username: str = ""
    employee_id: str = ""
    device_id: str = ""
    ip_address: str = ""
    mac_address: str = ""
    timestamp: str = ""
    department: str = ""
    custom_data: Dict[str, Any] = field(default_factory=dict)
    auto_fill: bool = True
    include_checksum: bool = True
    _checksum: str = field(default="", init=False)
    
    def __post_init__(self):
        """初始化后处理，自动填充设备信息"""
        if self.auto_fill:
            self._fill_device_info()
        if self.include_checksum:
            self._generate_checksum()
    
    def _fill_device_info(self):
        """自动填充设备信息"""
        collector = DeviceInfoCollector()
        
        if not self.ip_address:
            self.ip_address = collector.get_ip_address()
        
        if not self.mac_address:
            self.mac_address = collector.get_mac_address()
        
        if not self.timestamp:
            self.timestamp = collector.get_timestamp()
    
    def _generate_checksum(self):
        """生成校验码"""
        content = self.to_string(include_checksum=False)
        self._checksum = ChecksumGenerator.generate(content)
    
    @property
    def checksum(self) -> str:
        """获取校验码"""
        return self._checksum
    
    def verify_checksum(self, checksum: str) -> bool:
        """
        验证校验码
        
        Args:
            checksum: 待验证的校验码
            
        Returns:
            校验是否通过
        """
        content = self.to_string(include_checksum=False)
        return ChecksumGenerator.verify(content, checksum)
    
    def refresh_dynamic_info(self):
        """刷新动态信息（时间戳、设备状态等）"""
        collector = DeviceInfoCollector()
        self.timestamp = collector.get_timestamp()
        self.ip_address = collector.get_ip_address()
        if self.include_checksum:
            self._generate_checksum()
    
    def to_dict(self) -> Dict[str, Any]:
        """
        转换为字典
        
        Returns:
            包含所有信息的字典
        """
        data = {
            "username": self.username,
            "employee_id": self.employee_id,
            "device_id": self.device_id,
            "ip_address": self.ip_address,
            "mac_address": self.mac_address,
            "timestamp": self.timestamp,
            "department": self.department,
        }
        
        if self.custom_data:
            data["custom_data"] = self.custom_data
        
        if self.include_checksum and self._checksum:
            data["checksum"] = self._checksum
        
        return data
    
    def to_string(
        self, 
        separator: str = " | ", 
        include_checksum: bool = True,
        compact: bool = False
    ) -> str:
        """
        转换为字符串（用于水印显示）
        
        Args:
            separator: 字段分隔符
            include_checksum: 是否包含校验码
            compact: 是否使用紧凑格式
            
        Returns:
            格式化的水印字符串
        """
        parts = []
        
        if self.username:
            parts.append(f"用户:{self.username}" if not compact else self.username)
        
        if self.employee_id:
            parts.append(f"工号:{self.employee_id}" if not compact else self.employee_id)
        
        if self.department:
            parts.append(f"部门:{self.department}" if not compact else self.department)
        
        if self.device_id:
            parts.append(f"设备:{self.device_id}" if not compact else self.device_id)
        
        if self.ip_address:
            parts.append(f"IP:{self.ip_address}" if not compact else self.ip_address)
        
        if self.mac_address:
            parts.append(f"MAC:{self.mac_address}" if not compact else self.mac_address)
        
        if self.timestamp:
            parts.append(f"时间:{self.timestamp}" if not compact else self.timestamp)
        
        # 添加自定义数据
        for key, value in self.custom_data.items():
            parts.append(f"{key}:{value}")
        
        # 添加校验码
        if include_checksum and self._checksum:
            parts.append(f"校验:{self._checksum[:8]}" if not compact else self._checksum[:8])
        
        return separator.join(parts)
    
    def to_multiline_string(self) -> str:
        """
        转换为多行字符串
        
        Returns:
            多行格式的水印字符串
        """
        return self.to_string(separator="\n")
    
    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "WatermarkInfo":
        """
        从字典创建实例
        
        Args:
            data: 包含水印信息的字典
            
        Returns:
            WatermarkInfo实例
        """
        return cls(
            username=data.get("username", ""),
            employee_id=data.get("employee_id", ""),
            device_id=data.get("device_id", ""),
            ip_address=data.get("ip_address", ""),
            mac_address=data.get("mac_address", ""),
            timestamp=data.get("timestamp", ""),
            department=data.get("department", ""),
            custom_data=data.get("custom_data", {}),
            auto_fill=False,  # 从字典创建时不自动填充
            include_checksum=data.get("include_checksum", True),
        )
    
    @classmethod
    def from_string(cls, text: str, separator: str = " | ") -> "WatermarkInfo":
        """
        从字符串解析水印信息
        
        Args:
            text: 水印字符串
            separator: 字段分隔符
            
        Returns:
            WatermarkInfo实例
        """
        data = {}
        parts = text.split(separator)
        
        field_mapping = {
            "用户": "username",
            "工号": "employee_id",
            "部门": "department",
            "设备": "device_id",
            "IP": "ip_address",
            "MAC": "mac_address",
            "时间": "timestamp",
            "校验": "checksum",
        }
        
        for part in parts:
            if ":" in part:
                key, value = part.split(":", 1)
                key = key.strip()
                value = value.strip()
                
                if key in field_mapping:
                    data[field_mapping[key]] = value
        
        return cls.from_dict(data)
    
    def __str__(self) -> str:
        return self.to_string()
    
    def __repr__(self) -> str:
        return f"WatermarkInfo(username='{self.username}', employee_id='{self.employee_id}')"



"""
设备信息收集器 - 自动获取IP、MAC地址等设备信息
"""

import socket
import uuid
import platform
from datetime import datetime
from typing import Dict, Optional


class DeviceInfoCollector:
    """设备信息收集器"""
    
    @staticmethod
    def get_ip_address() -> str:
        """获取本机IP地址"""
        try:
            # 创建一个UDP socket来获取本机IP
            s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            s.connect(("8.8.8.8", 80))
            ip = s.getsockname()[0]
            s.close()
            return ip
        except Exception:
            try:
                return socket.gethostbyname(socket.gethostname())
            except Exception:
                return "127.0.0.1"
    
    @staticmethod
    def get_mac_address() -> str:
        """获取本机MAC地址"""
        try:
            mac = uuid.getnode()
            mac_str = ':'.join(('%012X' % mac)[i:i+2] for i in range(0, 12, 2))
            return mac_str
        except Exception:
            return "00:00:00:00:00:00"
    
    @staticmethod
    def get_hostname() -> str:
        """获取主机名"""
        try:
            return socket.gethostname()
        except Exception:
            return "unknown"
    
    @staticmethod
    def get_timestamp(format_str: str = "%Y-%m-%d %H:%M:%S") -> str:
        """获取当前时间戳"""
        return datetime.now().strftime(format_str)
    
    @staticmethod
    def get_platform_info() -> Dict[str, str]:
        """获取平台信息"""
        return {
            "system": platform.system(),
            "release": platform.release(),
            "version": platform.version(),
            "machine": platform.machine(),
            "processor": platform.processor(),
        }
    
    @classmethod
    def collect_all(cls) -> Dict[str, str]:
        """收集所有设备信息"""
        platform_info = cls.get_platform_info()
        return {
            "ip_address": cls.get_ip_address(),
            "mac_address": cls.get_mac_address(),
            "hostname": cls.get_hostname(),
            "timestamp": cls.get_timestamp(),
            "system": platform_info["system"],
            "machine": platform_info["machine"],
        }
